@@ -21,11 +21,9 @@
 
 #include "common_includes.h"
 
-RCSID("$Id: colors.c 280 2006-03-09 19:44:48Z bas $")
-
-#ifndef NO_COLOR_CURSES
+#ifdef HAVE_CURSES_COLOR
 extern struct colours cols;
-#endif /* NO_COLOR_CURSES */
+#endif /* HAVE_CURSES_COLOR */
 
 int normal;
 int menuselected;
@@ -44,11 +42,11 @@ int searchhighlight;
 void
 initcolors()
 {
-#ifndef NO_COLOR_CURSES
+#ifdef HAVE_CURSES_COLOR
 	if (has_colors())
 	{
 		start_color();
-#ifdef HAVE_USE_DEFAULT_COLORS
+#ifdef HAVE_DECL_USE_DEFAULT_COLORS
 		use_default_colors();
 #endif
 		normal = COLOR_PAIR(NORMAL);	/* normal text */
@@ -144,7 +142,7 @@ initcolors()
 	}
 	else
 	{
-#endif /* NO_COLOR_CURSES */
+#endif /* HAVE_CURSES_COLOR */
 		normal = A_NORMAL;
 		menu = A_BOLD;
 		note = A_BOLD;
@@ -158,7 +156,7 @@ initcolors()
 		manualitalic = A_BOLD;
 		infohighlight = A_BOLD;
 		searchhighlight = A_BOLD;
-#ifndef NO_COLOR_CURSES
+#ifdef HAVE_CURSES_COLOR
 	}
-#endif /* NO_COLOR_CURSES */
+#endif /* HAVE_CURSES_COLOR */
 }
