@@ -68,8 +68,8 @@ InfoHistory;
 
 typedef struct
 {
-	int line;			/* line number of the place where the link is */
-	int col;			/* column number ----||---- */
+	unsigned line;			/* line number of the place where the link is */
+	unsigned col;			/* column number ----||---- */
 	int breakpos;		/* col number, where the links breaks to next line */
 	int type;			/* type of link: 0 -  * menu::,
 						   1 -  * Comment: menu.
@@ -123,11 +123,11 @@ extern SearchAgain searchagain;
 
 /* an array of references for info */
 extern HyperObject *hyperobjects;
-extern int hyperobjectcount;
+extern unsigned long hyperobjectcount;
 /* an array of indirect entries [1 to n] */
 extern Indirect *indirect;
 /* number of indirect entries */
-extern int IndirectEntries;
+extern unsigned IndirectEntries;
 /* an array of tag table entries [1 to n] */
 extern TagTable *tag_table;
 /* offset of the first node in info file */
@@ -135,9 +135,9 @@ extern long FirstNodeOffset;
 /* name of the first node in info file */
 extern char FirstNodeName[256];
 /* number of tag table entries */
-extern int TagTableEntries;
+extern unsigned TagTableEntries;
 /* maximum dimensions of screen */
-extern int maxx, maxy;
+extern unsigned int maxx, maxy;
 extern InfoHistory infohistory;
 /* position to by set when moving via history */
 extern int npos;
@@ -179,12 +179,19 @@ extern int ClearScreenAtExit;
 /* determines whether when using readline wrapper to call the latest history
  * entry as default prompt or not */
 extern int CallReadlineHistory;
-
 /* quote ignored macros when watching page */
 extern int quote_ignored;
-
 /* set by SIGWINCH handler */
 extern int winchanged;
+/* true if we prefer man pages over info pages */
+extern int use_manual;
+/* true if pinfo/curses should grab the mouse and override normal terminal mouse behaviour */
+extern int grab_mouse;
+/* if true, search the current directory first, before checking /sur/share/info etc */
+extern int use_raw_filename;
+/* If set to true , pinfo will not attempt to display texinfo pages without tag tables */
+extern int DontHandleWithoutTagTable;
+
 
 /* initialize history (see struct above) * variables for `lastread' history */
 void inithistory ();
