@@ -22,8 +22,6 @@
 
 #include "common_includes.h"
 
-RCSID("$Id: filehandling_functions.c 298 2006-03-16 15:07:38Z bas $")
-
 typedef struct
 {
 	char *suffix;
@@ -31,8 +29,6 @@ typedef struct
 }
 Suffixes;
 
-char * basename(char *filename);
-
 
 /******************************************************************************
  * This piece of declarations says what to do with info files stored with      *
@@ -72,7 +68,7 @@ matchfile(char **buf, char *name)
 	char *bname = basename(name);
 	struct dirent *dp;
 	int matched = 0;
-	
+
 	/* remove a possible ".info" from the end of the file name
 	 * we're looking for */
 	strip_info_suffix(bname);
@@ -94,29 +90,40 @@ matchfile(char **buf, char *name)
 	/* iterate over all files in the directory */
 	while ((dp = readdir(dir)) != NULL)
 	{
-		/* use strcat rather than strdup, because xmalloc handles all 
+		/* use strcat rather than strdup, because xmalloc handles all
 		 * malloc errors */
-		char *thisfile = xmalloc(strlen(dp->d_name)+1);
-		strcat(thisfile, dp->d_name);
+		char *filename = xmalloc(strlen(dp->d_name)+1);
+		char *pagename = xmalloc(strlen(dp->d_name)+1);
+		strcat(filename, dp->d_name);
+		strcat(pagename, dp->d_name);
 
 		/* strip suffixes (so "gcc.info.gz" -> "gcc") */
-		strip_compression_suffix(thisfile);
-		strip_info_suffix(thisfile);
+		strip_compression_suffix(pagename);
+		strip_info_suffix(pagename);
+
+		/* strip compression suffix from returned filename
+		 * decompresison and type matching will happen later
+		 * (sigh)
+		 */
+		strip_compression_suffix(filename);
+
+		//fprintf(stdout,"Found filename `%s' (%s)\n", filename, pagename);
 
 		/* compare this file with the file we're looking for */
-		if (strcmp(thisfile,bname) == 0)
+		if (strcmp(pagename,bname) == 0)
 		{
 			/* we found a match! */
 			matched++;
 			/* put it in the buffer */
-			strncat(Buf, thisfile, 1023-strlen(Buf));
-			strncat(Buf, ".info", 1023-strlen(Buf));
+			strncat(Buf, filename, 1023-strlen(Buf));
 
 			/* clean up, and exit the loop */
-			xfree(thisfile);
+			xfree(filename);
+			xfree(pagename);
 			break;
 		}
-		xfree(thisfile);
+		xfree(filename);
+		xfree(pagename);
 	}
 	closedir(dir);
 
@@ -127,7 +134,7 @@ matchfile(char **buf, char *name)
 }
 
 FILE *
-dirpage_lookup(char **type, char ***message, long *lines,
+dirpage_lookup(char **type, char ***message, unsigned long *lines,
 		char *filename, char **first_node)
 {
 #define Type	(*type)
@@ -138,20 +145,18 @@ dirpage_lookup(char **type, char ***message, long *lines,
 	int goodHit = 0;
 	char name[256];
 	char file[256];
-	int i;
-	
+	char *nameend, *filestart, *fileend, *dot;
+
 	id = opendirfile(0);
 	if (!id)
-		return 0;
-	
+		return NULL;
+
 	read_item(id, type, message, lines);
 
-	char *nameend, *filestart, *fileend, *dot;
-	
 	/* search for node-links in every line */
-	for (i = 1; i < Lines; i++)
+	for (unsigned long i = 1; i < Lines; i++)
 	{
-		if ( (Message[i][0] == '*') && (Message[i][1] == ' ') 
+		if ( (Message[i][0] == '*') && (Message[i][1] == ' ')
 				&& ( nameend = strchr(Message[i], ':') )
 				&& (*(nameend + 1) != ':')	/* form: `* name:(file)node.' */
 				&& (filestart = strchr(nameend, '(') )
@@ -160,23 +165,24 @@ dirpage_lookup(char **type, char ***message, long *lines,
 				&& (strncasecmp(filename, Message[i] + 2, filenamelen) == 0)
 		   )
 		{
+			char *tmp;
 
-			/* skip this hit if it is not a perfect match and 
+			/* skip this hit if it is not a perfect match and
 			 * we have already found a previous partial match */
-			if ( ! ( (nameend - Message[i]) - 2 == filenamelen ) 
+			if ( ! ( (nameend - Message[i]) - 2 == filenamelen )
 					&&	goodHit )
 			{
 				continue;
 			}
 
 			/* find the name of the node link */
-			char *tmp = name;
+			tmp = name;
 			strncpy(file, filestart + 1, fileend - filestart - 1);
 			file[fileend - filestart - 1] = 0;
 			strncpy(name, fileend + 1, dot - fileend - 1);
 			name[dot - fileend - 1] = 0;
 			while (isspace(*tmp)) tmp++;
-			
+
 			if (strlen(name))
 			{
 				*first_node = xmalloc(strlen(tmp) + 1);
@@ -186,13 +192,13 @@ dirpage_lookup(char **type, char ***message, long *lines,
 			/* close the previously opened file */
 			if (id)
 			{
-				fclose(id);	
+				fclose(id);
 				id = 0;
 			}
 
 			/* see if this info file exists */
 			id = openinfo(file, 0);
-			if (id) 
+			if (id)
 			{
 				goodHit = 1;
 			}
@@ -200,7 +206,7 @@ dirpage_lookup(char **type, char ***message, long *lines,
 	}
 
 	/* if we haven't found anything, clean up and exit */
-	if (!goodHit)
+	if (id && !goodHit)
 	{
 		fclose(id);
 		id = 0;
@@ -214,12 +220,11 @@ dirpage_lookup(char **type, char ***message, long *lines,
 }
 
 void
-freeitem(char **type, char ***buf, long *lines)
+freeitem(char **type, char ***buf, unsigned long *lines)
 {
 #define Type	(*type)
 #define Buf		(*buf)
 #define Lines	(*lines)
-	long i;
 
 	if (Type != 0)
 	{
@@ -228,7 +233,7 @@ freeitem(char **type, char ***buf, long *lines)
 	}
 	if (Buf != 0)
 	{
-		for (i = 1; i <= Lines; i++)
+		for (unsigned long i = 1; i <= Lines; i++)
 			if (Buf[i] != 0)
 			{
 				xfree(Buf[i]);
@@ -243,7 +248,7 @@ freeitem(char **type, char ***buf, long *lines)
 }
 
 void
-read_item(FILE * id, char **type, char ***buf, long *lines)
+read_item(FILE * id, char **type, char ***buf, unsigned long *lines)
 {
 
 #define Type	(*type)
@@ -253,23 +258,26 @@ read_item(FILE * id, char **type, char ***buf, long *lines)
 
 	freeitem(type, buf, lines);	/* free previously allocated memory */
 
-	/* seek precisely on the INFO_TAG (the seeknode function may be imprecise
-	 * in combination with some weird tag_tables).  */
-	while (fgetc(id) != INFO_TAG);
-	/* then skip the trailing `\n' */
-	while (fgetc(id) != '\n');
-
-	/* allocate and read the header line */
-	Type = xmalloc(1024);
-	fgets(Type, 1024, id);
-	Type = xrealloc(Type, strlen(Type) + 1);
-
 	/* set number of lines to 0 */
 	Lines = 0;
 
 	/* initial buffer allocation */
 	Buf = xmalloc(sizeof(char **));
 
+	/* seek precisely on the INFO_TAG (the seeknode function may be imprecise
+	 * in combination with some weird tag_tables).  */
+	while (!feof(id) && fgetc(id) != INFO_TAG);
+	/* then skip the trailing `\n' */
+	while (!feof(id) && fgetc(id) != '\n');
+
+	/* allocate and read the header line */
+	Type = xmalloc(1024);
+	if (fgets(Type, 1024, id)==NULL)
+	{
+		/* nothing to do */
+		return;
+	}
+	Type = xrealloc(Type, strlen(Type) + 1);
 	/* now iterate over the lines */
 	do
 	{
@@ -283,6 +291,7 @@ read_item(FILE * id, char **type, char ***buf, long *lines)
 			Buf[Lines] = xrealloc(Buf[Lines], strlen(Buf[Lines]) + 1);
 		}
 
+		/* TODO: Weirdness going on here; looks like off-by-one error as Buf[0] is always "\0" */
 		/* increase the read lines number */
 		Lines++;
 
@@ -323,19 +332,19 @@ read_item(FILE * id, char **type, char ***buf, long *lines)
 	}
 
 	fseek(id, -2, SEEK_CUR);
+
 #undef Type
 #undef Buf
 #undef Lines
 
 }
 void
-load_indirect(char **message, long lines)
+load_indirect(char **message, unsigned long lines)
 {
-	long i;
 	char *wsk;
 	int cut = 0;			/* number of invalid entries */
 	indirect = xmalloc((lines + 1) * sizeof(Indirect));
-	for (i = 1; i < lines; i++)
+	for (unsigned long i = 1; i < lines; i++)
 	{
 		char *check;
 		wsk = message[i];
@@ -362,9 +371,8 @@ load_indirect(char **message, long lines)
 }
 
 void
-load_tag_table(char **message, long lines)
+load_tag_table(char **message, unsigned long lines)
 {
-	long i;
 	char *wsk, *wsk1;
 	int is_indirect = 0;
 	register unsigned int j;
@@ -378,7 +386,7 @@ load_tag_table(char **message, long lines)
 	if (strcasecmp("(Indirect)", message[1]) == 0)
 		is_indirect = 1;
 	tag_table = xmalloc((lines + 1) * sizeof(TagTable));
-	for (i = 1; i < lines - is_indirect; i++)
+	for (unsigned long i = 1; i < lines - is_indirect; i++)
 	{
 		char *check;
 		wsk = message[i + is_indirect];
@@ -420,7 +428,7 @@ load_tag_table(char **message, long lines)
 	/* FIXME: info should ALWAYS start at the 'Top' node, not at the first
 	   mentioned node(vide ocaml.info) */
 
-	for (i = 1; i <= TagTableEntries; i++)
+	for (unsigned int i = 1; i <= TagTableEntries; i++)
 	{
 		if (strcasecmp(tag_table[i].nodename, "Top") == 0)
 		{
@@ -431,6 +439,7 @@ load_tag_table(char **message, long lines)
 	qsort(&tag_table[1], TagTableEntries, sizeof(TagTable), qsort_cmp);
 }
 
+/* TODO: seek_indirect() and seek_tag_table() are almost identical: remove duplicate code */
 int
 seek_indirect(FILE * id)
 {
@@ -456,7 +465,16 @@ seek_indirect(FILE * id)
 			}
 		seek_pos = ftell(id) - 2;
 		fgetc(id);
-		fgets(type, 1024, id);
+		if (fgets(type, 1024, id)==0)
+		{
+			/* we're at the end of the file and haven't found any indirect refs. so bail out */
+			if (type)
+			{
+				xfree(type);
+				type = 0;
+			}
+			return 0;
+		}
 		if (strncasecmp("Indirect:", type, strlen("Indirect:")) == 0)
 		{
 			finito = 1;
@@ -525,12 +543,17 @@ seek_tag_table(FILE * id,int quiet)
 			}
 		}
 		seek_pos = ftell(id) - 2;
-		while (fgetc(id) != '\n')
+		while (fgetc(id) != '\n');
+		if (fgets(type, 1024, id)==NULL)
 		{
-			if (feof(id))
-				break;
+			/* we're at the end of the file and haven't found a tag table. so bail out */
+			if (type)
+			{
+				xfree(type);
+				type = 0;
+			}
+			return 2;
 		}
-		fgets(type, 1024, id);
 		if (strncasecmp("Tag Table:", type, strlen("Tag Table:")) == 0)
 		{
 			finito = 1;
@@ -551,7 +574,7 @@ seek_tag_table(FILE * id,int quiet)
 	return 1;
 }
 
-inline void
+void
 buildcommand(char *dest, char *command, char *filename, const char *tmpfilename)
 {
 	strcpy(dest, command);
@@ -561,7 +584,7 @@ buildcommand(char *dest, char *command, char *filename, const char *tmpfilename)
 	strcat(dest, tmpfilename);
 }
 
-inline void
+void
 builddircommand(char *dest, char *command, char *filename, const char *tmpfilename)
 {
 	strcpy(dest, command);
@@ -580,7 +603,7 @@ opendirfile(int number)
 	char command[1128];		/* holds command to evaluate for decompression of file */
 	int i, j;
 	char *tmpfilename = NULL;
-	int *fileendentries = xmalloc(infopathcount * sizeof(int));
+	size_t *fileendentries = xmalloc(infopathcount * sizeof(*fileendentries)); /* should really be off_t, but a signed type really doesn't make sense here */
 	int dir_found = 0;
 	int dircount = 0;
 	int lang_found;
@@ -618,7 +641,7 @@ opendirfile(int number)
 			{
 				fclose(id);
 				builddircommand(command, suffixes[j].command, buf, tmpfilename);
-				system(command);
+				xsystem(command);
 				lstat(tmpfilename, &status);
 				fileendentries[dircount] = status.st_size;
 				dircount++;
@@ -647,7 +670,7 @@ opendirfile(int number)
 				{
 					fclose(id);
 					builddircommand(command, suffixes[j].command, buf, tmpfilename);
-					system(command);
+					xsystem(command);
 					lstat(tmpfilename, &status);
 					fileendentries[dircount] = status.st_size;
 					dircount++;
@@ -666,7 +689,7 @@ opendirfile(int number)
 	if (id)
 	{
 		char *tmp;
-		long filelen, l;
+		size_t filelen, l;
 		int aswitch = 0;
 		int firstswitch = 0;
 		dircount = 0;
@@ -676,7 +699,12 @@ opendirfile(int number)
 
 		tmp = xmalloc(filelen);
 		fseek(id, 0, SEEK_SET);
-		fread(tmp, 1, filelen, id);
+		if (fread(tmp, 1, filelen, id)!=filelen)
+		{
+			printf(_("Error while reading file '%s'"), tmp);
+			closeprogram();
+			exit(1);
+		}
 		fclose(id);
 		id = fopen(tmpfilename, "w");
 		for (l = 0; l < filelen; l++)
@@ -709,20 +737,6 @@ opendirfile(int number)
 	return NULL;
 }
 
-char *
-basename(char *filename)
-{
-	int len = strlen(filename);
-	char *a = filename + len;
-	while (a > filename)
-	{
-		a--;
-		if (*a == '/')
-			return a + 1;
-	}
-	return filename;		/* when it was a basename */
-}
-
 /*
  * Note: openinfo is a function for reading info files, and putting
  * uncompressed content into a temporary filename.  For a flexibility, there
@@ -736,13 +750,14 @@ FILE *
 openinfo(char *filename, int number)
 {
 	FILE *id = NULL;
-	char *buf = xmalloc(1024);	/* holds local copy of filename */
+#define BUF_LEN 1024
+	char *buf = xmalloc(BUF_LEN);	/* holds local copy of filename */
 	char *bufend;			/* points at the trailing 0 of initial name */
 	char command[1128];		/* holds command to evaluate for decompression of file */
 	int i, j;
 	char *tmpfilename;
 
-	if (strncmp(filename, "dir", 3) == 0)
+	if ((strncmp(filename, "dir", 3)==0)  &&  !isalnum(filename[3]))
 	{
 		xfree(buf);
 		return opendirfile(number);
@@ -769,9 +784,9 @@ openinfo(char *filename, int number)
 		tmpfilename = tmpfilename2;	/* later we will refere only to tmp2 */
 	}
 
-	for (i = -1; i < infopathcount; i++)	/* go through all paths */
+	for (i = -2; i < infopathcount; i++)	/* go through all paths */
 	{
-		if (i == -1)
+		if (i < 0)
 		{
 			/*
 			 * no filenameprefix, we don't navigate around any specific
@@ -779,12 +794,12 @@ openinfo(char *filename, int number)
 			 */
 			if (!filenameprefix)
 				continue;
+			/* build a filename: First (i == -2) try filenameprefix/filename,
+			 * then try with a .info appended */
+			if (i == -2)
+				snprintf(buf, BUF_LEN, "%s/%s", filenameprefix, basename(filename));
 			else
-			{
-				strcpy(buf, filenameprefix);	/* build a filename */
-				strcat(buf, "/");
-				strcat(buf, basename(filename));
-			}
+				snprintf(buf, BUF_LEN, "%s/%s.info", filenameprefix, basename(filename));
 		}
 		else
 		{
@@ -816,7 +831,7 @@ openinfo(char *filename, int number)
 						}
 				}
 				buildcommand(command, suffixes[j].command, buf, tmpfilename);
-				system(command);
+				xsystem(command);
 				id = fopen(tmpfilename, "r");
 				if (id)
 				{
@@ -826,7 +841,7 @@ openinfo(char *filename, int number)
 			}
 			(*bufend) = 0;
 		}
-		
+
 		/* if we have a nonzero filename prefix, that is we view a set of
 		 * infopages, we don't want to search for a page in all
 		 * directories, but only in the prefix directory.  Therefore break
@@ -836,8 +851,9 @@ openinfo(char *filename, int number)
 	}
 	xfree(buf);
 
-	
+
 	return 0;
+#undef BUF_LEN
 }
 
 	void
@@ -944,7 +960,7 @@ initpaths()
 	while ( (next = strchr(dir, ':')) != NULL )
 	{
 		*next = '\0';  /* terminate the string */
-		
+
 		/* if the dir actually is a non-empty string, add it to paths[] */
 		if ( dir && strlen(dir)>0 )
 		{
@@ -1017,11 +1033,16 @@ initpaths()
 
 	/* ok, now we have all the (possibly) revelevant paths in paths[] */
 	/* now loop over them, see if they are valid and if they are duplicates*/
-	inodes = (ino_t *) xmalloc( maxpaths * sizeof(ino_t *) );
+	/* TODO: cleanup all malloc calls (get rid of cast, use sizeof(varname) instead of sizeof(type) */
+	inodes = (ino_t *) xmalloc( maxpaths * sizeof(ino_t) );
 	numpaths = 0;
 	len = 0;
 	for (i=0; i< maxpaths; i++)
 	{
+		/* TODO: check where these NULL paths come from */
+		if (paths[i]==NULL)
+			continue;
+
 		/* stat() the dir */
 		ret = stat( paths[i], &sbuf);
 		/* and see if it could be opened */
@@ -1094,19 +1115,34 @@ void
 create_indirect_tag_table()
 {
 	FILE *id = 0;
-	int i, j, initial;
-	for (i = 1; i <= IndirectEntries; i++)
+	int initial;
+	for (unsigned i = 1; i <= IndirectEntries; i++)
 	{
 		id = openinfo(indirect[i].filename, 1);
 		initial = TagTableEntries + 1;
-		if (id)
+		if (!id)
 		{
-			create_tag_table(id);
-			FirstNodeOffset = tag_table[1].offset;
-			strcpy(FirstNodeName, tag_table[1].nodename);
+			/* display error message to make the user aware of
+			 * the broken info page
+			 */
+			char msg[1024];
+			snprintf(msg, 1024, "%s '%s' (%s)",
+				_("Can't open file"), indirect[i].filename,
+				_("press a key to continue") );
+			attrset(bottomline);
+			mvhline(maxy - 1, 0, ' ', maxx);
+			mvaddstr(maxy - 1, 0, msg);
+			move(0, 0);
+			attrset(normal);
+			getch();
+
+			continue;
 		}
+		create_tag_table(id);
+		FirstNodeOffset = tag_table[1].offset;
+		strcpy(FirstNodeName, tag_table[1].nodename);
 		fclose(id);
-		for (j = initial; j <= TagTableEntries; j++)
+		for (unsigned j = initial; j <= TagTableEntries; j++)
 		{
 			tag_table[j].offset +=(indirect[i].offset - FirstNodeOffset);
 		}
@@ -1187,10 +1223,11 @@ create_tag_table(FILE * id)
 	}
 }
 
-void
+int
 seeknode(int tag_table_pos, FILE ** Id)
 {
 	int i;
+	FILE * newid;
 #define id	(*Id)
 	/*
 	 * Indirect nodes are seeked using a formula:
@@ -1204,15 +1241,17 @@ seeknode(int tag_table_pos, FILE ** Id)
 			if (indirect[i].offset <= tag_table[tag_table_pos].offset)
 			{
 				long off = tag_table[tag_table_pos].offset - indirect[i].offset + FirstNodeOffset - 4;
-				fclose(id);
-				id = openinfo(indirect[i].filename, 0);
-				if (id == NULL)
+				newid = openinfo(indirect[i].filename, 0);
+				if (newid == NULL)
 				{
+					return -1;
 					closeprogram();
-					printf(_("Error: could not open info file"));
+					printf(_("Error: could not open info file part"));
 					printf("\n");
 					exit(1);
 				}
+				fclose(id);
+				id = newid;
 				if (off > 0)
 					fseek(id, off, SEEK_SET);
 				else
@@ -1230,19 +1269,22 @@ seeknode(int tag_table_pos, FILE ** Id)
 			fseek(id, off, SEEK_SET);
 	}
 #undef id
+	return 0;
 }
 
 /* removes trailing .gz, .bz2, etc. */
 void
 strip_compression_suffix(char *file)
 {
+	const size_t len = strlen(file);
+	assert(len<1024); /* just some random limit */
 	char *found = 0;
-	int j;
-	for (j = 0; j < SuffixesNumber; j++)
+
+	for (unsigned j = 0; j < SuffixesNumber; j++)
 	{
 		if ( (found = strstr(file, suffixes[j].suffix)) != NULL )
 		{
-			if ( (file + strlen(file)) - found == strlen(suffixes[j].suffix) ) 
+			if ( file + len == found + strlen(suffixes[j].suffix) )
 			{
 				*found = '\0';
 				break;
@@ -1255,11 +1297,15 @@ strip_compression_suffix(char *file)
 void
 strip_info_suffix(char *file)
 {
+	const size_t len = strlen(file);
+	assert(len<1024); /* just some random limit */
+
 	char *found = 0;
-	char suffix[6] = ".info";
+	const char suffix[6] = ".info";
+
 	if ( (found = strstr(file, suffix)) != NULL )
 	{
-		if ( (file + strlen(file)) - found == strlen(suffix) )
+		if ( file + len == found + strlen(suffix) )
 		{
 			*found = '\0';
 		}
