@@ -20,8 +20,6 @@
  ***************************************************************************/
 #include "common_includes.h"
 
-RCSID("$Id: initializelinks.c 295 2006-03-15 21:52:38Z bas $")
-
 #define MENU_DOT 0
 #define NOTE_DOT 1
 
@@ -75,7 +73,7 @@ compare_tag_table_string(char *base, char *compared)
  * checks if an item belongs to tag table. returns 1 on success and 0 on
  * failure.  It should be optimised...
  */
-inline int
+int
 exists_in_tag_table(char *item)
 {
 	if (gettagtablepos(item) != -1)
@@ -97,6 +95,7 @@ freelinks()			/* frees space allocated previously by node-links */
 /*
  * Finds url end.  It is recognized by an invalid character.
  */
+/* TODO: fix possible string overflow (no bounds checking) */
 char *
 findurlend(char *str)
 {
@@ -110,12 +109,14 @@ findurlend(char *str)
 		if (*(end - 1) == '.')
 			end--;
 	}
+	assert(end>=str);
 	return end;
 }
 
 /*
  * Searchs for a note/menu delimiter.  it may be dot, comma, tab, or newline.
  */
+/* TODO: fix possible string overflow (no bounds checking) */
 char *
 finddot(char *str, int mynote)
 {
@@ -155,6 +156,7 @@ finddot(char *str, int mynote)
 		if ((end[i] < closest) &&(end[i]))
 			closest = end[i];
 	}
+	assert(closest>=str);
 	return closest;
 }
 
@@ -162,15 +164,14 @@ finddot(char *str, int mynote)
  * Moves you to the beginning of username in email address.  If username has
  * length=0, NULL is returned.
  */
+/* TODO: fix possible string overflow (no bounds checking) */
 char *
 findemailstart(char *str)
 {
 	char *allowedchars = "QWERTYUIOPASDFGHJKLZXCVBNMqwertyuiopasdfghjklzxcvbnm1234567890-_/~.%=|:";
 	char *at = strchr(str, '@');
-	char *emailstart = 0;
 	if (at)
 	{
-		emailstart = str;
 		while (at > str)
 		{
 			at--;
@@ -198,7 +199,7 @@ initializelinks(char *line1, char *line2, int line)
 	char *quotestart = 0, *quoteend = 0;
 	char *buf = xmalloc(strlen(line1) + strlen(line2) + 1);
 	/* required to sort properly the hyperlinks from current line only */
-	long initialhyperobjectcount = hyperobjectcount;
+	unsigned long initialhyperobjectcount = hyperobjectcount;
 	int changed;
 	int line1len = strlen(line1);
 
@@ -491,7 +492,7 @@ handlenote:
 		 *************************************************************************/
 		/* make sure that we don't handle notes, which fit in the second line */
 		/* Signed-unsigned issues FIXME */
-		if ((long)(notestart - buf) < strlen(line1))
+		if (notestart  < buf + strlen(line1))
 		{
 			/* we can handle only those, who are in the first line,
 			 * or who are split up into two lines */
@@ -517,12 +518,12 @@ handlenote:
 								end + 1, NodenameLen);
 						hyperobjects[hyperobjectcount].node[NodenameLen] = 0;
 						hyperobjects[hyperobjectcount].type = 2;
-						if (notestart + 7 - buf < strlen(line1))
+						if (notestart + 7 < buf + strlen(line1))
 						{
 							hyperobjects[hyperobjectcount].line = line;
 							hyperobjects[hyperobjectcount].col = calculate_len(buf, notestart + 7);
 							/* if the note highlight fits int first line */
-							if (tmp - buf < strlen(line1))
+							if (tmp < buf + strlen(line1))
 							{
 								/* we don't need to break highlighting
 								 * into several lines */
@@ -540,7 +541,7 @@ handlenote:
 							hyperobjects[hyperobjectcount].line = line + 1;
 							hyperobjects[hyperobjectcount].col =
 								calculate_len(buf + strlen(line1), notestart + 7);
-							if (tmp - buf < strlen(line1))	/* as above */
+							if (tmp < buf + strlen(line1))	/* as above */
 								hyperobjects[hyperobjectcount].breakpos = -1;
 							else if ((hyperobjects[hyperobjectcount].breakpos =
 										strlen(line1) -
@@ -569,13 +570,13 @@ handlenote:
 						strlen(hyperobjects[hyperobjectcount].node);
 					hyperobjects[hyperobjectcount].filelen =
 						strlen(hyperobjects[hyperobjectcount].file);
-					if (notestart + 7 - buf < strlen(line1))
+					if (notestart + 7 < buf + strlen(line1))
 					{
 						hyperobjects[hyperobjectcount].line = line;
 						hyperobjects[hyperobjectcount].col =
 							calculate_len(buf, notestart + 7) - 1;
 						/* if the note highlight fits int first line */
-						if (tmp - buf < strlen(line1))
+						if (tmp < buf + strlen(line1))
 						{
 							/* we don't need to break highlighting into
 							 * several lines */
@@ -593,7 +594,7 @@ handlenote:
 						hyperobjects[hyperobjectcount].line = line + 1;
 						hyperobjects[hyperobjectcount].col =
 							calculate_len(buf + strlen(line1), notestart + 7) - 1;
-						if (tmp - buf < strlen(line1))	/* as above */
+						if (tmp < buf + strlen(line1))	/* as above */
 							hyperobjects[hyperobjectcount].breakpos = -1;
 						else if ((hyperobjects[hyperobjectcount].breakpos =
 									strlen(line1)
@@ -650,12 +651,12 @@ handlenote:
 								end + 1, NodenameLen);
 						hyperobjects[hyperobjectcount].node[NodenameLen] = 0;
 						hyperobjects[hyperobjectcount].type = 3;
-						if (start - buf < strlen(line1))
+						if (start < buf + strlen(line1))
 						{
 							hyperobjects[hyperobjectcount].line = line;
 							hyperobjects[hyperobjectcount].col =
 								calculate_len(buf, start);
-							if (dot - buf < strlen(line1))
+							if (dot < buf + strlen(line1))
 							{
 								/* if the note highlight fits in first line
 								 * we don't need to break highlighting into
@@ -714,12 +715,12 @@ handle_no_file_note_label:
 							strlen(hyperobjects[hyperobjectcount].node);
 						hyperobjects[hyperobjectcount].filelen =
 							strlen(hyperobjects[hyperobjectcount].file);
-						if (start - buf < strlen(line1))
+						if (start < buf + strlen(line1))
 						{
 							hyperobjects[hyperobjectcount].line = line;
 							hyperobjects[hyperobjectcount].col =
 								calculate_len(buf, start);
-							if (dot - buf < strlen(line1))
+							if (dot < buf + strlen(line1))
 							{
 								/* if the note highlight fits in first line
 								 * we don't need to break highlighting into
