@@ -20,8 +20,6 @@
  ***************************************************************************/
 #include "common_includes.h"
 
-RCSID("$Id: mainfunction.c 298 2006-03-16 15:07:38Z bas $")
-
 #include <ctype.h>
 
 #ifndef MIN
@@ -38,10 +36,38 @@ int aftersearch = 0;
  * caused by the sequential auto-pgdn reading code
  */
 int toggled_by_menu = 0;
-long pos, cursor, infomenu, infocolumn=0;
+unsigned pos;
+long cursor;
+long infomenu, infocolumn=0;
+
+/* strdup src string, but when doing so, strip leading/trailing spaces and
+ * substitute multiple spaces with a single space. This is needed when *Node
+ * links span multiple lines and the margin is embedded in the string */
+static char *
+canonical_strdup(const char *src)
+{
+	char *dst;
+	int dstlen, srclen, copylen;
+
+	while (*src == ' ')
+		++src;
+	for (dstlen = 0, srclen = 0; src[srclen]; ++srclen)
+	{
+		if (src[srclen] != ' ' || (src[srclen + 1] != 0 && src[srclen + 1] != ' '))
+			++dstlen;
+	}
+	dst = xmalloc(dstlen + 1);
+	for (copylen = 0, srclen = 0; copylen < dstlen; ++srclen)
+	{
+		if (src[srclen] != ' ' || src[srclen + 1] != ' ')
+			dst[copylen++] = src[srclen];
+	}
+	dst[dstlen] = 0;
+	return dst;
+}
 
 WorkRVal
-work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
+work(char ***message, char **type, unsigned long *lines, FILE * id, int tag_table_pos)
 {
 #define Message	(*message)
 #define Lines	(*lines)
@@ -49,7 +75,7 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 	static WorkRVal rval =
 	{0, 0};
 	FILE *mypipe;
-	int i, fileoffset, j;
+	int fileoffset;
 	int indirectstart = -1;
 	int cursorchanged = 0;
 	int key = 0;
@@ -78,7 +104,7 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 #endif /*  getmaxyx */
 	/* free memory allocated previously by hypertext links */
 	freelinks();
-	for (i = 1; i < Lines; i++)	/* initialize node-links for every line */
+	for (unsigned long i = 1; i < Lines; i++)	/* initialize node-links for every line */
 	{
 		initializelinks(Message[i], Message[i + 1], i);
 	}
@@ -122,7 +148,7 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		if (key == ERR)
 		{
 			if (statusline == FREE)
-				showscreen(Message, Type, Lines, pos, cursor,infocolumn);
+				showscreen(Message, Lines, pos, cursor,infocolumn);
 			waitforgetch();
 			key = pinfo_getch();
 		}
@@ -188,7 +214,7 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 			if ((key == keys.goline_1) ||
 					(key == keys.goline_2))
 			{
-				long newpos;
+				unsigned long newpos;
 				attrset(bottomline);	/* read user's value */
 				move(maxy - 1, 0);
 				echo();
@@ -205,7 +231,7 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 							 */
 				{
 					int digit_val = 1;
-					for (i = 0; token[i] != 0; i++)
+					for (unsigned i = 0; token[i] != 0; i++)
 					{
 						if (!isdigit(token[i]))
 							digit_val = 0;
@@ -241,11 +267,11 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 				attrset(normal);
 
 				myendwin();
-				system("clear");
+				xsystem("clear");
 				mypipe = popen(token, "w");	/* open mypipe */
 				if (mypipe != NULL)
 				{
-					for (i = 1; i <= Lines; i++)	/* and flush the msg to stdin */
+					for (unsigned long i = 1; i <= Lines; i++)	/* and flush the msg to stdin */
 						fprintf(mypipe, "%s", Message[i]);
 					pclose(mypipe);
 					getchar();
@@ -318,7 +344,7 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 
 				/* Calculate current info file offset...  */
 				fileoffset = 0;
-				for (i = 1; i <= pos + 1; i++)	/* count the length of curnode */
+				for (unsigned i = 1; i <= pos + 1; i++)	/* count the length of curnode */
 					fileoffset += strlen(Message[i]);
 				fileoffset += strlen(Type);	/* add also header length */
 
@@ -335,7 +361,7 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 					long tokenpos;
 					long starttokenpos;
 					long filelen;
-					for (j = indirectstart; j <= IndirectEntries; j++)
+					for (int j = indirectstart; j <= (int) IndirectEntries; j++)
 					{
 						fd = openinfo(indirect[j].filename, 1);	/* get file
 																	 * length. */
@@ -355,8 +381,8 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 						starttokenpos = ftell(fd);
 
 						tmp = xmalloc(filelen - starttokenpos + 10);	/* read data */
-						fread(tmp, 1, filelen - starttokenpos, fd);
-						tmp[filelen - starttokenpos + 1] = 0;
+						size_t num_read = fread(tmp, 1, filelen - starttokenpos, fd);
+						tmp[num_read] = 0;
 
 						tokenpos = regexp_search(token, tmp);	/* search */
 
@@ -367,17 +393,16 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 							 * read to the memory
 							 */
 							tokenpos += starttokenpos;
-							{	/* local scope for tmpvar, matched */
-								int tmpvar = 0, matched = 0;
+							{	/* local scope for tmpvar */
+								int tmpvar = 0;
 								tag_table[0].offset = 0;
-								for (i = TagTableEntries; i >= 1; i--)
+								for (unsigned i = TagTableEntries; i >= 1; i--)
 								{
 									if ((tag_table[i].offset > tag_table[tmpvar].offset) &&
 											((tag_table[i].offset - indirect[j].offset + FirstNodeOffset) <= tokenpos))
 									{
 										return_value = i;
 										tmpvar = i;
-										matched = 1;
 									}
 								}
 							}
@@ -449,8 +474,8 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 
 					/* read data */
 					tmp = xmalloc(filelen - starttokenpos + 10);
-					fread(tmp, 1, filelen - starttokenpos, id);
-					tmp[filelen - starttokenpos + 1] = 0;
+					size_t num_read = fread(tmp, 1, filelen - starttokenpos, id);
+					tmp[num_read] = 0;
 
 					/* search */
 					tokenpos = regexp_search(token, tmp);
@@ -462,17 +487,16 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 						 * position.
 						 */
 						tokenpos += starttokenpos;
-						{		/* local scope for tmpvar, matched */
-							int tmpvar = 0, matched = 0;
+						{		/* local scope for tmpvar */
+							int tmpvar = 0;
 							tag_table[0].offset = 0;
-							for (i = TagTableEntries; i >= 1; i--)
+							for (unsigned i = TagTableEntries; i >= 1; i--)
 							{
 								if ((tag_table[i].offset > tag_table[tmpvar].offset) &&
 										(tag_table[i].offset <= tokenpos))
 								{
 									return_value = i;
 									tmpvar = i;
-									matched = 1;
 								}
 							}
 						}
@@ -533,8 +557,7 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 					infohistory.pos[infohistory.length] = pos;
 					infohistory.cursor[infohistory.length] = cursor;
 					infohistory.menu[infohistory.length] = infomenu;
-					rval.node = xmalloc(strlen(tag_table[return_value].nodename) + 1);
-					strcpy(rval.node, tag_table[return_value].nodename);
+					rval.node = canonical_strdup(tag_table[return_value].nodename);
 					rval.file = xmalloc(1);
 					rval.file[0] = 0;
 					return rval;
@@ -583,10 +606,10 @@ work(char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 					printw(_("Press any key to continue..."));
 					getch();
 					goto skip_search;
-					
+
 				}
 				/* scan for the token in the following lines.  */
-				for (i = pos + 1; i < Lines; i++)
+				for (unsigned long i = pos + 1; i < Lines; i++)
 				{
 					tmp = xmalloc(strlen(Message[i]) + strlen(Message[i + 1]) + 2);
 					/*
@@ -648,11 +671,18 @@ skip_search:
 				move(maxy - 1, 0);
 				attrset(bottomline);
 				curs_set(1);
-				token = getstring(_("Enter node name: "));	/* read user's wish */
+
+				const char ** tag_completions = completions_from_tag_table(tag_table, TagTableEntries);
+				token = getstring_with_completion(_("Enter node name: "), tag_completions);	/* read user's wish */
+				if (tag_completions!=NULL)
+				{
+					free(tag_completions);
+				}
+
 				curs_set(0);
 				noecho();
 				attrset(normal);
-				for (i = 1; i <= TagTableEntries; i++)
+				for (unsigned i = 1; i <= TagTableEntries; i++)
 				{
 					/* if the name was found in the tag table */
 					if (strcmp(token, tag_table[i].nodename) == 0)
@@ -668,8 +698,7 @@ skip_search:
 					infohistory.pos[infohistory.length] = pos;
 					infohistory.cursor[infohistory.length] = cursor;
 					infohistory.menu[infohistory.length] = infomenu;
-					rval.node = xmalloc(strlen(tag_table[return_value].nodename) + 1);
-					strcpy(rval.node, tag_table[return_value].nodename);
+					rval.node = canonical_strdup(tag_table[return_value].nodename);
 					rval.file = xmalloc(1);
 					rval.file[0] = 0;
 					aftersearch = 0;
@@ -699,8 +728,7 @@ skip_search:
 									break;
 								gotoendptr++;
 							}	/* skip spaces */
-							rval.node = xmalloc(strlen(gotoendptr) + 1);
-							strcpy(rval.node, gotoendptr);
+							rval.node = canonical_strdup(gotoendptr);
 							xfree(token);
 							token = 0;
 							aftersearch = 0;
@@ -747,8 +775,7 @@ skip_search:
 					infohistory.pos[infohistory.length] = pos;
 					infohistory.cursor[infohistory.length] = cursor;
 					infohistory.menu[infohistory.length] = infomenu;
-					rval.node = xmalloc(strlen(tag_table[return_value].nodename) + 1);
-					strcpy(rval.node, tag_table[return_value].nodename);
+					rval.node = canonical_strdup(tag_table[return_value].nodename);
 					rval.file = xmalloc(1);
 					rval.file[0] = 0;
 					aftersearch = 0;
@@ -769,8 +796,7 @@ skip_search:
 					infohistory.pos[infohistory.length] = pos;
 					infohistory.cursor[infohistory.length] = cursor;
 					infohistory.menu[infohistory.length] = infomenu;
-					rval.node = xmalloc(strlen(tag_table[return_value].nodename) + 1);
-					strcpy(rval.node, tag_table[return_value].nodename);
+					rval.node = canonical_strdup(tag_table[return_value].nodename);
 					rval.file = xmalloc(1);
 					rval.file[0] = 0;
 					aftersearch = 0;
@@ -798,8 +824,7 @@ skip_search:
 						infohistory.cursor[infohistory.length] = cursor;
 						infohistory.menu[infohistory.length] = infomenu;
 					}
-					rval.node = xmalloc(strlen(tag_table[return_value].nodename) + 1);
-					strcpy(rval.node, tag_table[return_value].nodename);
+					rval.node = canonical_strdup(tag_table[return_value].nodename);
 					rval.file = xmalloc(1);
 					rval.file[0] = 0;
 					aftersearch = 0;
@@ -824,7 +849,7 @@ skip_search:
 						 * look if there's a cursor(link) pos available above,
 						 * and if it is visible now.
 						 */
-						for (i = cursor - 1; i >= 0; i--)
+						for (int i = cursor - 1; i >= 0; i--)
 						{
 							if ((hyperobjects[i].line >= pos) &&
 									(hyperobjects[i].line < pos +(maxy - 1)))
@@ -844,7 +869,7 @@ skip_search:
 					if (pos > 1)	/* lower the nodepos */
 						pos--;
 					/* and scan for a hyperlink in the new line */
-					for (i = 0; i < hyperobjectcount; i++)
+					for (unsigned long i = 0; i < hyperobjectcount; i++)
 					{
 						if (hyperobjects[i].line == pos)
 						{
@@ -922,8 +947,10 @@ skip_search:
 					(key == keys.down_2))	/* top+bottom line \|/ */
 			{
 				cursorchanged = 0;	/* works similar to keys.up */
-				if (cursor < hyperobjectcount)
-					for (i = cursor + 1; i < hyperobjectcount; i++)
+				if (cursor < 0) cursor = -1;
+				if (cursor < 0 || (unsigned long) cursor < hyperobjectcount)
+				{
+					for (unsigned long i = (unsigned long) (cursor + 1); i < hyperobjectcount; i++)
 					{
 						if ((hyperobjects[i].line >= pos) &&
 								(hyperobjects[i].line < pos +(maxy - 2)))
@@ -936,11 +963,12 @@ skip_search:
 							}
 						}
 					}
+				}
 				if (!cursorchanged)
 				{
 					if (pos <= Lines -(maxy - 2))
 						pos++;
-					for (i = cursor + 1; i < hyperobjectcount; i++)
+					for (unsigned long i = cursor + 1; i < hyperobjectcount; i++)
 					{
 						if ((hyperobjects[i].line >= pos) &&
 								(hyperobjects[i].line < pos +(maxy - 2)))
@@ -962,8 +990,7 @@ skip_search:
 				infohistory.pos[infohistory.length] = pos;
 				infohistory.cursor[infohistory.length] = cursor;
 				infohistory.menu[infohistory.length] = infomenu;
-				rval.node = xmalloc(strlen(FirstNodeName) + 1);
-				strcpy(rval.node, FirstNodeName);
+				rval.node = canonical_strdup(FirstNodeName);
 				rval.file = xmalloc(1);
 				rval.file[0] = 0;
 				aftersearch = 0;
@@ -978,8 +1005,7 @@ skip_search:
 					dellastinfohistory();	/* remove history entry for this node */
 					/* now we deal with the previous node history entry */
 
-					rval.node = xmalloc(strlen(infohistory.node[infohistory.length]) + 1);
-					strcpy(rval.node, infohistory.node[infohistory.length]);
+					rval.node = canonical_strdup(infohistory.node[infohistory.length]);
 					rval.file = xmalloc(strlen(infohistory.file[infohistory.length]) + 1);
 					strcpy(rval.file, infohistory.file[infohistory.length]);
 
@@ -1000,17 +1026,16 @@ skip_search:
 				infohistory.menu[infohistory.length] = infomenu;
 				if (!toggled_by_menu)
 					infohistory.menu[infohistory.length] = cursor;
-				if ((cursor >= 0) &&(cursor < hyperobjectcount))
-					if ( 
-						( (hyperobjects[cursor].line >= pos) 
+				if ((cursor >= 0) && ((unsigned long) cursor < hyperobjectcount))
+					if (
+						( (hyperobjects[cursor].line >= pos)
 						  && (hyperobjects[cursor].line < pos +(maxy - 2) )
 						) || (toggled_by_menu) )
 					{
 						toggled_by_menu = 0;
 						if (hyperobjects[cursor].type < 4)	/* normal info link */
 						{
-							rval.node = xmalloc(strlen(hyperobjects[cursor].node) + 1);
-							strcpy(rval.node, hyperobjects[cursor].node);
+							rval.node = canonical_strdup(hyperobjects[cursor].node);
 							rval.file = xmalloc(strlen(hyperobjects[cursor].file) + 1);
 							strcpy(rval.file, hyperobjects[cursor].file);
 							aftersearch = 0;
@@ -1025,7 +1050,7 @@ skip_search:
 								strcat(tempbuf, " ");
 								strcat(tempbuf, hyperobjects[cursor].node);
 								myendwin();
-								system(tempbuf);
+								xsystem(tempbuf);
 								doupdate();
 								xfree(tempbuf);
 							}
@@ -1036,7 +1061,7 @@ skip_search:
 								strcat(tempbuf, " ");
 								strcat(tempbuf, hyperobjects[cursor].node);
 								myendwin();
-								system(tempbuf);
+								xsystem(tempbuf);
 								doupdate();
 								xfree(tempbuf);
 							}
@@ -1047,8 +1072,8 @@ skip_search:
 								strcat(tempbuf, " ");
 								strcat(tempbuf, hyperobjects[cursor].node);
 								myendwin();
-								system("clear");
-								system(tempbuf);
+								xsystem("clear");
+								xsystem(tempbuf);
 								doupdate();
 								xfree(tempbuf);
 							}
@@ -1075,18 +1100,26 @@ skip_search:
 			{
 				MEVENT mouse;
 				int done = 0;
+
 				getmouse(&mouse);
+				if (mouse.x<0 || mouse.y<0) /* should never happen, according to curses docs */
+					continue;
+
+				/* copy to unsigned vars to avoid all kinds of signed/unsigned comparison unpleasantness below */
+				unsigned mouse_x = mouse.x;
+				unsigned mouse_y = mouse.x;
+
 				if (mouse.bstate == BUTTON1_CLICKED)
 				{
-					if ((mouse.y > 0) &&(mouse.y < maxy - 1))
+					if ((mouse_y > 0) &&(mouse_y < maxy - 1))
 					{
-						for (i = cursor; i > 0; i--)
+						for (int i = cursor; i > 0; i--)
 						{
-							if (hyperobjects[i].line == mouse.y + pos - 1)
+							if (hyperobjects[i].line == mouse_y + pos - 1)
 							{
-								if (hyperobjects[i].col <= mouse.x - 1)
+								if (hyperobjects[i].col <= mouse_x - 1)
 								{
-									if (hyperobjects[i].col + strlen(hyperobjects[i].node) + strlen(hyperobjects[i].file) >= mouse.x - 1)
+									if (hyperobjects[i].col + strlen(hyperobjects[i].node) + strlen(hyperobjects[i].file) >= mouse_x - 1)
 									{
 										if (hyperobjects[i].type < HIGHLIGHT)
 										{
@@ -1099,13 +1132,13 @@ skip_search:
 							}
 						}
 						if (!done)
-							for (i = cursor; i < hyperobjectcount; i++)
+							for (unsigned i = cursor; i < hyperobjectcount; i++)
 							{
-								if (hyperobjects[i].line == mouse.y + pos - 1)
+								if (hyperobjects[i].line == mouse_y + pos - 1)
 								{
-									if (hyperobjects[i].col <= mouse.x - 1)
+									if (hyperobjects[i].col <= mouse_x - 1)
 									{
-										if (hyperobjects[i].col + strlen(hyperobjects[i].node) + strlen(hyperobjects[i].file) >= mouse.x - 1)
+										if (hyperobjects[i].col + strlen(hyperobjects[i].node) + strlen(hyperobjects[i].file) >= mouse_x - 1)
 										{
 											if (hyperobjects[i].type < HIGHLIGHT)
 											{
@@ -1117,23 +1150,23 @@ skip_search:
 									}
 								}
 							}
-					}		/* end: if (mouse.y not on top/bottom line) */
-					else if (mouse.y == 0)
+					}		/* end: if (mouse_y not on top/bottom line) */
+					else if (mouse_y == 0)
 						ungetch(keys.up_1);
-					else if (mouse.y == maxy - 1)
+					else if (mouse_y == maxy - 1)
 						ungetch(keys.down_1);
 				}		/* end: button clicked */
 				if (mouse.bstate == BUTTON1_DOUBLE_CLICKED)
 				{
-					if ((mouse.y > 0) &&(mouse.y < maxy - 1))
+					if ((mouse_y > 0) &&(mouse_y < maxy - 1))
 					{
-						for (i = cursor; i >= 0; i--)
+						for (int i = cursor; i >= 0; i--)
 						{
-							if (hyperobjects[i].line == mouse.y + pos - 1)
+							if (hyperobjects[i].line == mouse_y + pos - 1)
 							{
-								if (hyperobjects[i].col <= mouse.x - 1)
+								if (hyperobjects[i].col <= mouse_x - 1)
 								{
-									if (hyperobjects[i].col + strlen(hyperobjects[i].node) + strlen(hyperobjects[i].file) >= mouse.x - 1)
+									if (hyperobjects[i].col + strlen(hyperobjects[i].node) + strlen(hyperobjects[i].file) >= mouse_x - 1)
 									{
 										if (hyperobjects[i].type < HIGHLIGHT)
 										{
@@ -1146,13 +1179,13 @@ skip_search:
 							}
 						}
 						if (!done)
-							for (i = cursor; i < hyperobjectcount; i++)
+							for (unsigned i = cursor; i < hyperobjectcount; i++)
 							{
-								if (hyperobjects[i].line == mouse.y + pos - 1)
+								if (hyperobjects[i].line == mouse_y + pos - 1)
 								{
-									if (hyperobjects[i].col <= mouse.x - 1)
+									if (hyperobjects[i].col <= mouse_x - 1)
 									{
-										if (hyperobjects[i].col + strlen(hyperobjects[i].node) + strlen(hyperobjects[i].file) >= mouse.x - 1)
+										if (hyperobjects[i].col + strlen(hyperobjects[i].node) + strlen(hyperobjects[i].file) >= mouse_x - 1)
 										{
 											if (hyperobjects[i].type < HIGHLIGHT)
 											{
@@ -1166,10 +1199,10 @@ skip_search:
 							}
 						if (done)
 							ungetch(keys.followlink_1);
-					}		/* end: if (mouse.y not on top/bottom line) */
-					else if (mouse.y == 0)
+					}		/* end: if (mouse_y not on top/bottom line) */
+					else if (mouse_y == 0)
 						ungetch(keys.pgup_1);
-					else if (mouse.y == maxy - 1)
+					else if (mouse_y == maxy - 1)
 						ungetch(keys.pgdn_1);
 				}		/* end: button doubleclicked */
 			}
@@ -1194,13 +1227,12 @@ skip_search:
 void
 next_infomenu()
 {
-	int i;
 	if (hyperobjectcount == 0)
 	{
 		infomenu = -1;
 		return;
 	}
-	for (i = infomenu + 1; i < hyperobjectcount; i++)
+	for (unsigned long i = infomenu + 1; i < hyperobjectcount; i++)
 	{
 		if (hyperobjects[i].type <= 1)	/* menu item */
 		{
@@ -1214,8 +1246,7 @@ next_infomenu()
 void
 rescan_cursor()
 {
-	int i;
-	for (i = 0; i < hyperobjectcount; i++)
+	for (unsigned long i = 0; i < hyperobjectcount; i++)
 	{
 		if ((hyperobjects[i].line >= pos) &&
 				(hyperobjects[i].line < pos +(maxy - 2)))
