@@ -20,8 +20,6 @@
  ***************************************************************************/
 #include "common_includes.h"
 
-RCSID("$Id: manual.c 296 2006-03-16 14:07:22Z bas $")
-
 #include <ctype.h>
 #include <sys/stat.h>
 #include <stdlib.h>
@@ -58,13 +56,13 @@ void printmanual(char **Message, long Lines);
 /* line by line stored manual */
 char **manual = 0;
 /* number of lines in manual */
-int ManualLines = 0;
+unsigned ManualLines = 0;
 int selected = -1;		/* number of selected link(offset in 'manuallinks',
 						   bellow) */
-int manualpos = 0;		/* number of the first line, which is painted on
+unsigned manualpos = 0;	/* number of the first line, which is painted on
 						   screen */
 
-int manualcol = 0;		/* the first displayed column of manpage--
+unsigned manualcol = 0;	/* the first displayed column of manpage--
 						   for moving the screen left/right */
 
 int manual_aftersearch = 0;	/* this is set if man page is now after search
@@ -75,7 +73,7 @@ int manwidthChanged = 0;	/* this flag indicates whether the env variable
 typedef struct
 {
 	/* name of a manual */
-	char name[256];
+	char name[128];
 	/* section */
 	char sect[32];
 	/* what was last selected on this page */
@@ -96,9 +94,9 @@ int manualhistorylength = 0;
 /* this structure describes a hyperlink in manual viewer */
 typedef struct
 {			/* struct for hypertext references */
-	int line;		/* line of the manpage, where the reference is */
+	unsigned int line;		/* line of the manpage, where the reference is */
 	/* column of that line */
-	int col;
+	unsigned int col;
 	/* name of the reference */
 	char *name;
 	/* section of the reference */
@@ -113,7 +111,7 @@ manuallink;
 manuallink *manuallinks = 0;
 
 /* number of found manual references in man page */
-int ManualLinks = 0;
+unsigned ManualLinks = 0;
 
 /* semaphore for checking if it's a history(left arrow) call */
 int historical = 0;
@@ -123,7 +121,7 @@ void
 /* free buffers allocated by current man page */
 manual_free_buffers()
 {
-	int i;
+	unsigned int i;
 	/* first free previously allocated memory */
 	/* for the manual itself... */
 	if (manual)
@@ -175,8 +173,38 @@ set_initial_history(char *name)
 
 	/* filename->name */
 	strcpy(manualhistory[0].name, &name1[i]);
-	/* section unknown */
-	strcpy(manualhistory[0].sect, "");
+
+	/* It is essential to know the section name, as otherwise links in the
+	 * man page to the same name but different section would be ignored -
+	 * see man_initializelinks. An example is sleep(1) which has link to
+	 * sleep(3). So we try to find the section directly from man. */
+	if (manualhistory[0].sect[0] == 0) {
+		char buf[1024];
+		char *str, *lastSlash, *lastButOneSlash;
+		FILE *pathFile;
+		snprintf(buf, sizeof(buf), "man -w -W %s %s", ManOptions, name);
+		pathFile = popen(buf, "r");
+		if (fgets(buf, sizeof(buf), pathFile)==NULL)
+		{
+			fprintf(stderr, "Error executing command '%s'\n", buf);
+			exit(1);
+		}
+		pclose(pathFile);
+		/* buf will be of the form "/usr/share/man/man1/sleep.1.gz". We
+		 * find the section from the leaf directory "/man1" */
+		for (str = buf, lastSlash = str, lastButOneSlash = 0; *str; ++str) {
+			if (*str == '/') {
+				lastButOneSlash = lastSlash;
+				lastSlash = str;
+			}
+		}
+		if (lastButOneSlash) {
+			*lastSlash = 0; /* terminate the section */
+			lastButOneSlash += 4; /* skip "/man", and land on the section */
+			strncpy(manualhistory[0].sect, lastButOneSlash, sizeof(manualhistory[0].sect)-1);
+		}
+	}
+
 	/* selected unknown */
 	manualhistory[0].selected = -1;
 	/* pos=0 */
@@ -203,7 +231,7 @@ construct_manualname(char *buf, int which)
 			char *base = xmalloc(1024);
 			char *ptr;
 			int tmppos;
-			strcpy(base, manual[manuallinks[which].line - 1]);
+			strncpy(base, manual[manuallinks[which].line - 1],1023);
 			strip_manual(base);
 			ptr = base + strlen(base) - 3;
 			while (((isalpha(*ptr)) ||(*ptr == '.') ||(*ptr == '_')) &&(ptr > base))
@@ -213,9 +241,8 @@ construct_manualname(char *buf, int which)
 				ptr++;
 			strcpy(buf, ptr);
 			tmppos = strlen(buf);
-			/* TODO: check the following statement */
-			if (tmppos > 1);
-			buf[tmppos - 2] = 0;
+			if (tmppos > 1)
+				buf[tmppos - 2] = 0;
 			strcat(buf, manuallinks[which].name);
 			xfree(base);
 		}
@@ -251,7 +278,7 @@ handlemanual(char *name)
 	FILE *id;
 
 	char manualname[256];
-	char cmd[256];
+	char cmd[4096];
 	char *raw_tempfilename = 0;
 	char *apropos_tempfilename = 0;
 
@@ -326,7 +353,7 @@ handlemanual(char *name)
 				{
 					printf(_("Apropos pages:\n"));
 					snprintf(cmd, 255, "apropos %s|cat %s", name, StderrRedirection);
-					system(cmd);
+					xsystem(cmd);
 				}
 				return 1;
 			}
@@ -443,12 +470,15 @@ handlemanual(char *name)
 	 ****************************************************************************/
 #endif
 	if (!plain_apropos)
+	{
 		snprintf(cmd, 255, "man %s %s %s > %s",
 				ManOptions,
 				name,
 				StderrRedirection,
 				tmpfilename1);
-	if ((plain_apropos) ||(system(cmd) != 0))
+	}
+
+	if (plain_apropos || (system_check(cmd) != 0))
 	{
 		if (!plain_apropos)
 		{
@@ -460,10 +490,10 @@ handlemanual(char *name)
 		{
 			printf(_("Calling apropos \n"));
 			apropos_tempfilename = make_tempfile();
-			snprintf(cmd, 255, "apropos %s > %s", name, apropos_tempfilename);
-			if (system(cmd) != 0)
+			snprintf(cmd, 4096, "apropos %s > %s", name, apropos_tempfilename);
+			if (system_check(cmd) != 0)
 			{
-				printf(_("Nothing appropiate\n"));
+				printf(_("Nothing appropriate\n"));
 				unlink(apropos_tempfilename);
 				return 1;
 			}
@@ -514,7 +544,7 @@ handlemanual(char *name)
 			if (return_value != -2)
 			{
 				construct_manualname(manualname, return_value);
-				snprintf(cmd, 255, "man %s %s %s %s > %s",
+				snprintf(cmd, 4096, "man %s %s %s %s > %s",
 						ManOptions,
 						manuallinks[return_value].section,
 						manualname,
@@ -551,13 +581,13 @@ handlemanual(char *name)
 				 */
 				historical = 1;
 			}
-			system(cmd);
+			xsystem(cmd);
 			stat(tmpfilename2, &statbuf);
 			if (statbuf.st_size > 0)
 			{
 				snprintf(cmd, 255, "mv %s %s", tmpfilename2, tmpfilename1);
 				/* create tmp file containing man page */
-				system(cmd);
+				xsystem(cmd);
 				/* open man page */
 				id = fopen(tmpfilename1, "r");
 				if (id != NULL)
@@ -709,7 +739,7 @@ man_initializelinks(char *tmp, int carry)
 	int tmpcnt = strlen(tmp) + 1;
 	char *mylink = tmp;
 	char *urlstart, *urlend;
-	long initialManualLinks = ManualLinks;
+	unsigned initialManualLinks = ManualLinks;
 	int i, b;
 	/******************************************************************************
 	 * handle url refrences                                                       *
@@ -797,7 +827,7 @@ man_initializelinks(char *tmp, int carry)
 				if ((!strchr(p_t1, '(')) &&(!is_in_manlinks(manlinks, p_t1)))
 				{
 					char tempchar;
-					int breakpos;
+					int breakpos, cols_before_link;
 					i = mylink - tmp - 1;
 					if (i < 0)
 						i++;
@@ -830,7 +860,7 @@ man_initializelinks(char *tmp, int carry)
 					 */
 
 					/* calculate the number of columns in front of the link */
-					int cols_before_link = width_of_string(tmp, i-1);
+					cols_before_link = width_of_string(tmp, i-1);
 
 					/* a small check */
 					if (!((use_apropos) &&(manualhistorylength == 0)))
@@ -910,7 +940,7 @@ manualwork()
 	/* key, which contains the value entered by user */
 	int key = 0;
 	/* tmp values */
-	int i, selectedchanged;
+	int selectedchanged;
 	int statusline = FREE;
 #ifdef getmaxyx
 	/* if ncurses, get maxx and maxy */
@@ -967,7 +997,7 @@ manualwork()
 			key = pinfo_getch();
 		}
 		/************************ keyboard handling **********************************/
-		if (key != 0)
+		if (key > 0)
 		{
 			if ((key == keys.print_1) ||
 					(key == keys.print_2))
@@ -990,7 +1020,7 @@ manualwork()
 				curs_set(0);
 				noecho();
 				move(maxy - 1, 0);
-#ifdef HAVE_BKGDSET
+#ifdef HAVE_DECL_BKGDSET
 				bkgdset(' ' | bottomline);
 				clrtoeol();
 				bkgdset(0);
@@ -1021,7 +1051,7 @@ manualwork()
 				curs_set(0);
 				noecho();
 				move(maxy - 1, 0);
-#ifdef HAVE_BKGDSET
+#ifdef HAVE_DECL_BKGDSET
 				bkgdset(' ' | bottomline);
 				clrtoeol();
 				bkgdset(0);
@@ -1033,7 +1063,7 @@ manualwork()
 				if (token)
 				{
 					int digit_val = 1;
-					for (i = 0; token[i] != 0; i++)
+					for (unsigned i = 0; token[i] != 0; i++)
 					{
 						if (!isdigit(token[i]))
 							digit_val = 0;
@@ -1043,7 +1073,7 @@ manualwork()
 					{
 						newpos = atol(token);
 						newpos -=(maxy - 1);
-						if ((newpos >= 0) &&(newpos < ManualLines -(maxy - 2)))
+						if ((newpos >= 0) && ((unsigned long) newpos < ManualLines - (maxy - 2)))
 							manualpos = newpos;
 						else if (newpos > 0)
 							manualpos = ManualLines -(maxy - 2);
@@ -1067,7 +1097,7 @@ manualwork()
 				token = getstring(_("Enter command: "));
 				noecho();
 				move(maxy - 1, 0);
-#ifdef HAVE_BKGDSET
+#ifdef HAVE_DECL_BKGDSET
 				bkgdset(' ' | bottomline);
 				clrtoeol();
 				bkgdset(0);
@@ -1077,13 +1107,13 @@ manualwork()
 				attrset(normal);
 
 				myendwin();
-				system("clear");
+				xsystem("clear");
 				/* open mypipe */
 				mypipe = popen(token, "w");
 				if (mypipe != NULL)
 				{
 					/* and flush the msg to stdin */
-					for (i = 0; i < ManualLines; i++)
+					for (unsigned i = 0; i < ManualLines; i++)
 						fprintf(mypipe, "%s", manual[i]);
 					pclose(mypipe);
 				}
@@ -1135,7 +1165,7 @@ manualwork()
 				curs_set(0);
 				noecho();
 				move(maxy - 1, 0);
-#ifdef HAVE_BKGDSET
+#ifdef HAVE_DECL_BKGDSET
 				bkgdset(' ' | bottomline);
 				clrtoeol();
 				bkgdset(0);
@@ -1159,7 +1189,7 @@ manualwork()
 					goto skip_search;
 				}
 				/* and search for it in all subsequential lines */
-				for (i = manualpos + 1; i < ManualLines - 1; i++)
+				for (unsigned i = manualpos + 1; i < ManualLines - 1; i++)
 				{
 					char *tmp;
 					tmp = xmalloc(strlen(manual[i]) + strlen(manual[i + 1]) + 10);
@@ -1235,7 +1265,7 @@ skip_search:
 						 * scan for a next visible one, which is above the
 						 * current.
 						 */
-						for (i = selected - 1; i >= 0; i--)
+						for (int i = selected - 1; i >= 0; i--)
 						{
 							if ((manuallinks[i].line >= manualpos) &&
 									(manuallinks[i].line < manualpos +(maxy - 1)))
@@ -1253,7 +1283,7 @@ skip_search:
 					if (manualpos >= 1)
 						manualpos--;
 					/* and scan for selected again :) */
-					for (i = 0; i < ManualLinks; i++)
+					for (unsigned i = 0; i < ManualLinks; i++)
 					{
 						if (manuallinks[i].line == manualpos)
 						{
@@ -1267,16 +1297,18 @@ skip_search:
 			if ((key == keys.end_1) ||
 					(key == keys.end_2))
 			{
-				manualpos = ManualLines -(maxy - 1);
-				if (manualpos < 0)
+				if (ManualLines < maxy - 1)
 					manualpos = 0;
+				else
+					manualpos = ManualLines -(maxy - 1);
+
 				selected = ManualLinks - 1;
 			}
 			/*=====================================================*/
 			if ((key == keys.nextnode_1) ||
 					(key == keys.nextnode_2))
 			{
-				for (i = manualpos + 1; i < ManualLines; i++)
+				for (unsigned i = manualpos + 1; i < ManualLines; i++)
 				{
 					if (manual[i][1] == 8)
 					{
@@ -1289,7 +1321,7 @@ skip_search:
 			if ((key == keys.prevnode_1) ||
 					(key == keys.prevnode_2))
 			{
-				for (i = manualpos - 1; i > 0; i--)
+				for (unsigned i = manualpos - 1; i > 0; i--)
 				{
 					if (manual[i][1] == 8)
 					{
@@ -1347,8 +1379,21 @@ skip_search:
 			if ((key == keys.down_1) || (key == keys.down_2))
 			{
 				selectedchanged = 0;
-				if (selected < ManualLinks)
-					for (i = selected + 1; i < ManualLinks; i++)
+				for (unsigned i = selected + 1; i < ManualLinks; i++)
+				{
+					if ((manuallinks[i].line >= manualpos) &&
+							(manuallinks[i].line < manualpos +(maxy - 2)))
+					{
+						selected = i;
+						selectedchanged = 1;
+						break;
+					}
+				}
+				if (!selectedchanged)
+				{
+					if (manualpos < ManualLines -(maxy - 1))
+						manualpos++;
+					for (unsigned i = selected + 1; i < ManualLinks; i++)
 					{
 						if ((manuallinks[i].line >= manualpos) &&
 								(manuallinks[i].line < manualpos +(maxy - 2)))
@@ -1358,21 +1403,6 @@ skip_search:
 							break;
 						}
 					}
-				if (!selectedchanged)
-				{
-					if (manualpos < ManualLines -(maxy - 1))
-						manualpos++;
-					if (selected < ManualLinks)
-						for (i = selected + 1; i < ManualLinks; i++)
-						{
-							if ((manuallinks[i].line >= manualpos) &&
-									(manuallinks[i].line < manualpos +(maxy - 2)))
-							{
-								selected = i;
-								selectedchanged = 1;
-								break;
-							}
-						}
 				}
 			}
 			/*=====================================================*/
@@ -1401,7 +1431,7 @@ skip_search:
 							buflen = strlen(tempbuf);
 							construct_manualname(tempbuf + buflen, selected);
 							myendwin();
-							system(tempbuf);
+							xsystem(tempbuf);
 							doupdate();
 							xfree(tempbuf);
 						}
@@ -1414,7 +1444,7 @@ skip_search:
 							buflen = strlen(tempbuf);
 							construct_manualname(tempbuf + buflen, selected);
 							myendwin();
-							system(tempbuf);
+							xsystem(tempbuf);
 							doupdate();
 							xfree(tempbuf);
 						}
@@ -1427,7 +1457,7 @@ skip_search:
 							buflen = strlen(tempbuf);
 							construct_manualname(tempbuf + buflen, selected);
 							myendwin();
-							system(tempbuf);
+							xsystem(tempbuf);
 							doupdate();
 							xfree(tempbuf);
 						}
@@ -1451,17 +1481,24 @@ skip_search:
 				MEVENT mouse;
 				int done = 0;
 				getmouse(&mouse);
+				if (mouse.x<0 || mouse.y<0) /* should never happen, according to curses docs */
+					continue;
+
+				/* copy to unsigned vars to avoid all kinds of signed/unsigned comparison unpleasantness below */
+				unsigned mouse_x = mouse.x;
+				unsigned mouse_y = mouse.x;
+
 				if (mouse.bstate == BUTTON1_CLICKED)
 				{
-					if ((mouse.y > 0) &&(mouse.y < maxy - 1))
+					if ((mouse_y > 0) &&(mouse_y < maxy - 1))
 					{
-						for (i = selected; i >= 0; i--)
+						for (int i = selected; i >= 0; i--)
 						{
-							if (manuallinks[i].line == mouse.y + manualpos - 1)
+							if (manuallinks[i].line == mouse_y + manualpos - 1)
 							{
-								if (manuallinks[i].col <= mouse.x - 1)
+								if (manuallinks[i].col <= mouse_x - 1)
 								{
-									if (manuallinks[i].col + strlen(manuallinks[i].name) >= mouse.x - 1)
+									if (manuallinks[i].col + strlen(manuallinks[i].name) >= mouse_x - 1)
 									{
 										selected = i;
 										done = 1;
@@ -1471,13 +1508,13 @@ skip_search:
 							}
 						}
 						if (!done)
-							for (i = selected; i < ManualLinks; i++)
+							for (unsigned i = selected; i < ManualLinks; i++)
 							{
-								if (manuallinks[i].line == mouse.y + manualpos - 1)
+								if (manuallinks[i].line == mouse_y + manualpos - 1)
 								{
-									if (manuallinks[i].col <= mouse.x - 1)
+									if (manuallinks[i].col <= mouse_x - 1)
 									{
-										if (manuallinks[i].col + strlen(manuallinks[i].name) >= mouse.x - 1)
+										if (manuallinks[i].col + strlen(manuallinks[i].name) >= mouse_x - 1)
 										{
 											selected = i;
 											done = 1;
@@ -1487,22 +1524,22 @@ skip_search:
 								}
 							}
 					}		/* end: mouse not on top/bottom line */
-					if (mouse.y == 0)
+					if (mouse_y == 0)
 						ungetch(keys.up_1);
-					if (mouse.y == maxy - 1)
+					if (mouse_y == maxy - 1)
 						ungetch(keys.down_1);
 				}		/* end: button_clicked */
 				if (mouse.bstate == BUTTON1_DOUBLE_CLICKED)
 				{
-					if ((mouse.y > 0) &&(mouse.y < maxy - 1))
+					if ((mouse_y > 0) &&(mouse_y < maxy - 1))
 					{
-						for (i = selected; i >= 0; i--)
+						for (int i = selected; i >= 0; i--)
 						{
-							if (manuallinks[i].line == mouse.y + manualpos - 1)
+							if (manuallinks[i].line == mouse_y + manualpos - 1)
 							{
-								if (manuallinks[i].col <= mouse.x - 1)
+								if (manuallinks[i].col <= mouse_x - 1)
 								{
-									if (manuallinks[i].col + strlen(manuallinks[i].name) >= mouse.x - 1)
+									if (manuallinks[i].col + strlen(manuallinks[i].name) >= mouse_x - 1)
 									{
 										selected = i;
 										done = 1;
@@ -1512,13 +1549,13 @@ skip_search:
 							}
 						}
 						if (!done)
-							for (i = selected; i < ManualLinks; i++)
+							for (unsigned i = selected; i < ManualLinks; i++)
 							{
-								if (manuallinks[i].line == mouse.y + manualpos - 1)
+								if (manuallinks[i].line == mouse_y + manualpos - 1)
 								{
-									if (manuallinks[i].col <= mouse.x - 1)
+									if (manuallinks[i].col <= mouse_x - 1)
 									{
-										if (manuallinks[i].col + strlen(manuallinks[i].name) >= mouse.x - 1)
+										if (manuallinks[i].col + strlen(manuallinks[i].name) >= mouse_x - 1)
 										{
 											selected = i;
 											done = 1;
@@ -1530,9 +1567,9 @@ skip_search:
 						if (done)
 							ungetch(keys.followlink_1);
 					}		/* end: mouse not at top/bottom line */
-					if (mouse.y == 0)
+					if (mouse_y == 0)
 						ungetch(keys.pgup_1);
-					if (mouse.y == maxy - 1)
+					if (mouse_y == maxy - 1)
 						ungetch(keys.pgdn_1);
 				}		/* end: button doubleclicked */
 			}
@@ -1558,8 +1595,7 @@ void
 /* scan for some hyperlink, available on current screen */
 rescan_selected()
 {
-	int i;
-	for (i = 0; i < ManualLinks; i++)
+	for (unsigned i = 0; i < ManualLinks; i++)
 	{
 		if ((manuallinks[i].line >= manualpos) &&
 				(manuallinks[i].line < manualpos +(maxy - 1)))
@@ -1587,16 +1623,15 @@ char *getmancolumn(char *man, int mancol)
 void
 showmanualscreen()
 {
-	int i;
 #ifdef getmaxyx
 	/* refresh maxy, maxx values */
 	getmaxyx(stdscr, maxy, maxx);
 #endif
 	attrset(normal);
 	/* print all visible text lines */
-	for (i = manualpos;(i < manualpos +(maxy - 2)) &&(i < ManualLines); i++)
+	for (unsigned i = manualpos;(i < manualpos +(maxy - 2)) &&(i < ManualLines); i++)
 	{
-		int len = strlen(manual[i]);
+		size_t len = strlen(manual[i]);
 		if (len)
 			manual[i][len - 1] = ' ';
 		/* if we have something to display */
@@ -1611,12 +1646,12 @@ showmanualscreen()
 		if (len)
 			manual[i][len - 1] = '\n';
 	}
-#ifdef HAVE_BKGDSET
+#ifdef HAVE_DECL_BKGDSET
 	bkgdset(' ' | normal);
 #endif
 	/* and clear to bottom */
 	clrtobot();
-#ifdef HAVE_BKGDSET
+#ifdef HAVE_DECL_BKGDSET
 	bkgdset(0);
 #endif
 	attrset(normal);
@@ -1691,7 +1726,7 @@ label_check_italic:
 		}
 label_skip_other:;
 	}
-#ifdef HAVE_BKGDSET
+#ifdef HAVE_DECL_BKGDSET
 	bkgdset(' ' | normal);
 	clrtoeol();
 	bkgdset(0);
@@ -1737,7 +1772,7 @@ add_highlights()
 {
 	int i;
 	/* scan through the visible objects */
-	for (i = 0; i < ManualLinks; i++)
+	for (i = 0; (unsigned) i < ManualLinks; i++)
 	{
 		/* if the object is on the current screen */
 		if ((manuallinks[i].line >= manualpos) &&
@@ -1757,7 +1792,7 @@ add_highlights()
 					int x, y, ltline = manuallinks[i].line - 1;
 					/* find the line, where starts the split link */
 					char *tmpstr = strdup(manual[ltline]);
-					int ltlinelen;
+					size_t ltlinelen;
 					char *newlinemark;
 					/* remove boldfaces&italics */
 					strip_manual(tmpstr);
@@ -1814,19 +1849,22 @@ add_highlights()
 					 * to the line defined in manlinks(line+1)
 					 */
 					char *tmpstr = strdup(manual[ltline]);
-					char *wsk = tmpstr, *wskend;
+					unsigned long k = 0;
+					char *wskend;
 					strip_manual(tmpstr);
 					/* skip spaces */
-					while (isspace(*wsk))
-						wsk++;
+					while (isspace(tmpstr[k]))
+						k++;
 					/* find the end of url */
-					wskend = findurlend(wsk);
+					wskend = findurlend(tmpstr+k);
+					if (wskend<tmpstr) abort(); /* TODO: crude check, but this should never occur anyway */
+
 					/* add end of string, and print */
 					*wskend = 0;
-					if (wsk-tmpstr<manualcol)
-						mvaddstr(manuallinks[i].line - manualpos + 2, wsk - tmpstr - manualcol, wsk);
-					else if (wskend-tmpstr<manualcol)
-						mvaddstr(manuallinks[i].line - manualpos + 2, 0, wsk+manualcol);
+					if (k<manualcol)
+						mvaddstr(manuallinks[i].line - manualpos + 2, k - manualcol, tmpstr+k);
+					else if ((uint64_t) (wskend-tmpstr)<manualcol) /* should be safe see check above */
+						mvaddstr(manuallinks[i].line - manualpos + 2, 0, tmpstr+k+manualcol);
 				}
 			}
 			if (manuallinks[i].col>manualcol)
