@@ -21,27 +21,15 @@
 
 #include "common_includes.h"
 
-RCSID("$Id: parse_config.c 280 2006-03-09 19:44:48Z bas $")
-
 #include <ctype.h>
 
-#define COLOR_DEFAULT -1	/* mutt uses this was for transparency */
+#ifdef HAVE_DECL_USE_DEFAULT_COLORS
+# define COLOR_DEFAULT (-1)	/* ncurses extension to use default color, see default_colors(3NCURSES) */
+#endif
 
 regex_t *h_regexp = 0;	/* regexps to highlight */
 int h_regexp_num = 0;	/* number of those regexps */
 
-extern int use_apropos;
-extern int CutManHeaders;
-extern int DontHandleWithoutTagTable;
-extern int use_manual;
-extern int use_raw_filename;
-extern int quote_ignored;
-extern char *httpviewer;
-extern char *ftpviewer;
-extern char *maileditor;
-extern char *manlinks;
-extern char *ignoredmacros;
-
 struct keybindings keys =
 {
 	's',		'S',		/* regexp search */
@@ -51,11 +39,11 @@ struct keybindings keys =
 	'n',		'N',		/* next node */
 	'u',		'U',		/* up node */
 	KEY_UP,		'k',		/* up one line */
-#ifdef HAVE_KEY_END
+#ifdef KEY_END
 	KEY_END
-#else	/* HAVE_KEY_END */
+#else
 	'E'
-#endif	/* HAVE_KEY_END */
+#endif
 		,'e',			/* end */
 	KEY_NPAGE,	' ',		/* down one page */
 	KEY_HOME,	'H',		/* home */
@@ -79,7 +67,7 @@ struct keybindings keys =
 	'6',		0			/* scroll right */
 };
 
-#ifndef NO_COLOR_CURSES
+#ifdef HAVE_CURSES_COLOR
 struct colours cols =
 {
 	COLOR_WHITE,	COLOR_BLACK,	NO_BOLD,	NO_BLINK,	/* normal */
@@ -96,7 +84,7 @@ struct colours cols =
 	COLOR_WHITE,	COLOR_BLACK,	BOLD,		NO_BLINK,	/* info highlight(quoted text) */
 	COLOR_YELLOW,	COLOR_BLACK,	BOLD,		NO_BLINK	/* search highlight */
 };
-#endif /* NO_COLOR_CURSES */
+#endif /* HAVE_CURSES_COLOR */
 
 int
 parse_config(void)
@@ -311,7 +299,7 @@ parse_line(char *line)
 		else
 			return 1;
 	}
-#ifndef NO_COLOR_CURSES
+#ifdef HAVE_CURSES_COLOR
 	else if (!strncmp(temp, "COL", 3))
 	{
 		key = NULL;
@@ -409,7 +397,7 @@ parse_line(char *line)
 		else
 			return 1;
 	}
-#endif /* NO_COLOR_CURSES */
+#endif /* HAVE_CURSES_COLOR */
 	else if (!strncmp(temp, "MANUAL", 6))
 	{
 		temp = strtok(NULL, "=");
@@ -425,6 +413,21 @@ parse_line(char *line)
 				return 1;
 		}
 	}
+	else if (!strncmp(temp, "GRAB-MOUSE", 10))
+	{
+		temp = strtok(NULL, "=");
+		if (temp)
+		{
+			if (!(temp = str_toupper(skip_whitespace(temp))))
+				return 1;
+			if (!strncmp(temp, "TRUE", 4))
+				grab_mouse = 1;
+			else if (!strncmp(temp, "FALSE", 5))
+				grab_mouse = 0;
+			else
+				return 1;
+		}
+	}
 	else if (!strncmp(temp, "RAW-FILENAME", 12))
 	{
 		temp = strtok(NULL, "=");
@@ -601,7 +604,7 @@ parse_line(char *line)
 		else
 			return 1;
 	}
-	else if (!strncmp(temp, "FTPVIEWER", 10))
+	else if (!strncmp(temp, "FTPVIEWER", 9))
 	{
 		temp = strtok(NULL, "\n");
 		if (temp)
@@ -769,7 +772,7 @@ parse_line(char *line)
 	}
 	else
 		return 1;
-#ifndef NO_COLOR_CURSES
+#ifdef HAVE_CURSES_COLOR
 	if (fore)
 	{
 		for (i = 0; i < 4; i++)
@@ -805,7 +808,14 @@ parse_line(char *line)
 			else if (!(strncmp(temp, "COLOR_MAGENTA", 13)))
 				*p = COLOR_MAGENTA;
 			else if (!(strncmp(temp, "COLOR_DEFAULT", 13)))
+			{
+#ifdef HAVE_DECL_USE_DEFAULT_COLORS
 				*p = COLOR_DEFAULT;
+#else
+				fprintf(stderr, "COLOR_DEFAULT is not supported on this system\n");
+				return 1;
+#endif
+			}
 			else if (!(strncmp(temp, "BOLD", 4)))
 				*p = BOLD;
 			else if (!(strncmp(temp, "NO_BOLD", 7)))
@@ -819,7 +829,7 @@ parse_line(char *line)
 		}
 	}
 	else
-#endif /* NO_COLOR_CURSES */
+#endif /* HAVE_CURSES_COLOR */
 		if (key)
 		{
 			if (!(temp = skip_whitespace(strtok(NULL, "="))))
@@ -976,7 +986,7 @@ skip_whitespace(char *str)
 char *
 remove_quotes(char *str)
 {
-	int i = 0;
+	size_t i = 0;
 
 	for (i = 0; i < strlen(str); i++)
 		if (str[i] == '\"')
