@@ -21,14 +21,11 @@
 
 #include "common_includes.h"
 
-RCSID(PKG_VER "$Id: pinfo.c 280 2006-03-09 19:44:48Z bas $")
-
 #ifdef HAVE_GETOPT_LONG
  #include <getopt.h>
 #endif
 
-char *version = VERSION;
-int DontHandleWithoutTagTable = 0;
+char *version = PACKAGE_VERSION;
 
 /* currently viewed filename */
 char *curfile = 0;
@@ -55,7 +52,7 @@ main(int argc, char *argv[])
 	int command_line_option;
 	FILE *id = NULL;
 	/* line count in message */
-	long lines = 0;
+	unsigned long lines = 0;
 	/* this will hold node's text */
 	char **message = 0;
 	/* this will hold the node's header */
@@ -118,8 +115,8 @@ main(int argc, char *argv[])
 				 */
 				for (i = 1; i < argc; i++)
 				{
-					strcat(filename, argv[i]);
-					strcat(filename, " ");
+					strncat(filename, argv[i], sizeof(filename)-strlen(filename)-2);
+					strncat(filename, " ", 2);
 				}
 				exit(handlemanual(filename));
 			}
@@ -136,7 +133,7 @@ main(int argc, char *argv[])
 		{
 			char *tmp;
 			command_line_option = getopt_long(argc, argv,
-					"hvmfrapcsdtnlx", long_options, NULL);
+					"hvmfrapcsdtn:lx", long_options, NULL);
 			switch(command_line_option)
 			{
 				case 'x':
@@ -199,8 +196,8 @@ main(int argc, char *argv[])
 					strcpy(filename, "");
 					for (i = optind; i < argc; i++)
 					{
-						strcat(filename, argv[i]);
-						strcat(filename, " ");
+						strncat(filename, argv[i], sizeof(filename)-strlen(filename)-2);
+						strncat(filename, " ", 2);
 					}
 					exit(handlemanual(filename));
 				case 'f':
@@ -239,6 +236,8 @@ main(int argc, char *argv[])
 				case 's':
 					CutEmptyManLines = 1;
 					break;
+				case '?':
+					exit(1);
 			}
 		}
 		while (command_line_option != EOF);
@@ -378,32 +377,39 @@ main(int argc, char *argv[])
 		do
 		{
 			/* set seek offset for given node */
-			seeknode(tag_table_pos, &id);
+			if (seeknode(tag_table_pos, &id)!=0)
+			{
+				tag_table_pos = gettagtablepos(FirstNodeName);
+				if (seeknode(tag_table_pos, &id)!=0)
+				{
+					exit(-1);
+				}
+			}
 			/* read the node */
 			read_item(id, &type, &message, &lines);
 
 			/* handle goto/link where no file was found -- see below */
 			if (!filenotfound)
 			{
-				addinfohistory(curfile, tag_table[tag_table_pos].nodename, 
+				addinfohistory(curfile, tag_table[tag_table_pos].nodename,
 						-1, -1, -1);
 			}
 			else
 				filenotfound = 0;
 
 			/* this might have been allocated in the previous iteration */
-			if (file_name_force!=NULL) 
+			if (file_name_force!=NULL)
 			{
 				xfree(file_name_force);
 				file_name_force = NULL;
 			}
 
 			/* check if we really found the node we were looking for
-			 * (don't do this for tag tables we manually created, as this 
+			 * (don't do this for tag tables we manually created, as this
 			 * might cause a loop if somethign goes wrong)
 			 *
-			 * the entire handling of file_name_force and work_return_value.file is a 
-			 * big hack, but it'll have to do for now, until the entire work 
+			 * the entire handling of file_name_force and work_return_value.file is a
+			 * big hack, but it'll have to do for now, until the entire work
 			 * loop thing is rewritten.
 			 */
 			if ( (ForceManualTagTable==0)
@@ -411,11 +417,11 @@ main(int argc, char *argv[])
 			{
 				/* Oops, we found the wrong node! */
 
-				/* display error message to make the user aware of 
-				 * the broken info page 
+				/* display error message to make the user aware of
+				 * the broken info page
 				 */
 				char msg[81];
-				snprintf(msg, 81, "%s (%s)", 
+				snprintf(msg, 81, "%s (%s)",
 						_("Tag table is corrupt, trying to fix..."),
 						_("press a key to continue") );
 				attrset(bottomline);
@@ -425,13 +431,13 @@ main(int argc, char *argv[])
 				attrset(normal);
 				getch();
 
-				/* We found another node than we were looking for, so the 
+				/* We found another node than we were looking for, so the
 				 * tag table must be corrupt. Try to fix it by manually
 				 * creating tag tables and... */
 				ForceManualTagTable = 1;
-				
-				/* forcing the current file to reload by seting 
-				 * work_return_value.file to the current file, and the current 
+
+				/* forcing the current file to reload by seting
+				 * work_return_value.file to the current file, and the current
 				 * file to \0
 				 */
 				if (file_name_force) xfree(file_name_force);
@@ -443,7 +449,7 @@ main(int argc, char *argv[])
 				/* remove this try from the history stack */
 				dellastinfohistory();
 			}
-			else 
+			else
 			{
 				/* everything went fine, so display the node and wait for
 				 * key events and stuff */
@@ -476,6 +482,13 @@ main(int argc, char *argv[])
 						tmp = strdup(work_return_value.file);
 						clearfilenameprefix();
 						id = openinfo(tmp, 0);
+
+						/* try to lookup the name in dir file */
+						if (id == NULL)
+						{
+							id = dirpage_lookup(&type, &message, &lines, tmp, &pinfo_start_node);
+						}
+
 						xfree(tmp);
 						tmp = 0;
 						/* if the file doesn't exist */
@@ -634,6 +647,7 @@ checksu()
 {
 	struct passwd *pswd;
 	struct group *grwd;
+	int result = 0;
 
 	if (!getegid() || !getgid())
 	{
@@ -649,13 +663,13 @@ checksu()
 		else
 		{
 			if (!getgid() && !getuid())
-				setgid(grwd->gr_gid);
+				result = setgid(grwd->gr_gid);
 			else
-				setegid(grwd->gr_gid);
+				result = setegid(grwd->gr_gid);
 		}
 	}
 
-	if (!geteuid() || !getuid())
+	if (result==0 && (!geteuid() || !getuid()) )
 	{
 		pswd = getpwnam(safe_user);
 		if (!pswd)
@@ -669,10 +683,16 @@ checksu()
 		else
 		{
 			if (!getuid())
-				setuid(pswd->pw_uid);
+				result = setuid(pswd->pw_uid);
 			else
-				seteuid(pswd->pw_uid);
+				result = seteuid(pswd->pw_uid);
 		}
 	}
 
+	if (result != 0)
+	{
+		printf(_("Unable to drop root privileges: %s"), strerror(errno));
+		exit(-1);
+	}
+
 }
