@@ -21,22 +21,19 @@
 
 #include "common_includes.h"
 
-RCSID("$Id: printinfo.c 280 2006-03-09 19:44:48Z bas $")
-
 /*
  * Algorithm: We first print highlights, then we send `\r' to the printer,
  * and we draw the base line. Thus highlights are printed `twice', and
  * are darker than the rest :)
  */
 void
-printnode(char ***message, long *lines)
+printnode(char ***message, unsigned long *lines)
 {
 #define Message	(*message)
 #define Lines	(*lines)
 
 	/* counter, to point at what highlights are already * handled */
-	int highlight = 0;
-	int i, j;
+	unsigned highlight = 0;
 	/* printer fd */
 	FILE *prnFD;
 	/* temporary buffer */
@@ -45,7 +42,7 @@ printnode(char ***message, long *lines)
 	prnFD = popen(printutility, "w");
 
 	/* scan through all lines */
-	for (i = 1; i < Lines; i++)
+	for (unsigned i = 1; i < Lines; i++)
 	{
 		/*
 		 * this says, where the printer's head is
@@ -82,7 +79,7 @@ printnode(char ***message, long *lines)
 			}
 			else if (hyperobjects[highlight].line == i)
 			{
-				for (j = 0; j < hyperobjects[highlight].col - lineprinted; j++)
+				for (unsigned j = 0; j < hyperobjects[highlight].col - lineprinted; j++)
 					fprintf(prnFD, " ");
 				fprintf(prnFD, "%s", buf);
 				lineprinted = hyperobjects[highlight].col +
