@@ -19,13 +19,11 @@
  *  USA
  ***************************************************************************/
 
-#include"common_includes.h"
-
-RCSID("$Id: regexp_search.c 280 2006-03-09 19:44:48Z bas $")
+#include "common_includes.h"
 
 #ifndef ___DONT_USE_REGEXP_SEARCH___
-#include"regex.h"
-#include<ctype.h>
+#include "regex.h"
+#include <ctype.h>
 /* adapted partialy from midnight commander view regexp search */
 
 enum
@@ -74,7 +72,7 @@ __regexp_search(char *pattern, char *string)
 		/* invalid regexp */
 		if (regcomp(&h_regexp[pinfo_re_offset], pattern, flags))
 		{
-			return 0;
+			return -1;
 		}
 		old_pattern = strdup(pattern);
 		old_type = match_type;
