@@ -21,8 +21,6 @@
 
 #include "common_includes.h"
 
-RCSID("$Id: utils.c 296 2006-03-16 14:07:22Z bas $")
-
 #include <regex.h>
 #include <ctype.h>
 #include <sys/select.h>
@@ -34,7 +32,7 @@ RCSID("$Id: utils.c 296 2006-03-16 14:07:22Z bas $")
 char *safe_user = "nobody";
 char *safe_group = "nogroup";
 
-#ifndef HAVE_CURS_SET
+#ifndef HAVE_DECL_CURS_SET
 void
 curs_set(int a)
 {
@@ -50,7 +48,6 @@ int pinfo_re_offset = -1;
 #ifdef HAS_READLINE
 #include <readline/readline.h>
 #include <readline/history.h>
-#include <term.h>
 /* HAS_READLINE */
 #endif
 
@@ -107,6 +104,7 @@ xfree(void *ptr)
 	free(ptr);
 }
 
+/* TODO: get rid of this xmalloc nonsense */
 void *
 xmalloc(size_t size)
 {
@@ -193,7 +191,33 @@ xrealloc(void *ptr, size_t size)
 	return value;
 }
 
-inline void
+int
+system_check(const char *command)
+{
+	if (command==NULL)
+	{
+		return -1;
+	}
+	int result = system(command);
+	if (WIFEXITED(result))
+	{
+		return WEXITSTATUS(result);
+	}
+	return -1;
+}
+
+void
+xsystem(const char *command)
+{
+	int result = system_check(command);
+	if (result!=0)
+	{
+		printf(_("Failed to execute command '%s': %i"), command, result);
+		exit(2);
+	}
+}
+
+void
 initlocale()
 {
 #ifdef ___DEBUG___
@@ -202,7 +226,6 @@ initlocale()
 		malloc_addr[i] = 0;
 /* ___DEBUG___ */
 #endif
-	sbrk(100000);
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
@@ -239,34 +262,135 @@ checkfilename(char *filename)
 void
 my_rl_display()
 {
-	/* go to the bottom line, empty it, and print the prompt and buffer */
+	static size_t len = 0;
+
+	/* if the user's input has changed, clear the entire line to remove possible leftover completions */
+	size_t newlen = strlen(rl_line_buffer);
+	if (newlen!=len)
+	{
+		mymvhline(maxy - 1, 0, ' ', maxx);
+		len = newlen;
+	}
+
+	/* go to the bottom line, print the prompt and buffer */
 	attrset(bottomline);
-	mymvhline(maxy - 1, 0, ' ', maxx);
 	move(maxy-1,0);
+
 	printw("%s%s", rl_prompt, rl_line_buffer);
 	refresh();
 }
+
+void
+my_rl_completion_display(char **matches, int num_matches, int UNUSED(max_length))
+{
+	if (num_matches<1)
+	{
+		return;
+	}
+	/* redraw entire prompt line, appended with possible matches */
+	move(maxy-1,0);
+	printw("%s", rl_prompt);
+	/* note: first entry is the entered text, matches start at index 1 */
+	printw("%s  ", matches[0]);
+	for (int i=1; i < num_matches+1; i++)
+	{
+		printw("%s ", matches[i]);
+	}
+	/* and return prompt to correct position */
+	move(maxy-1, strlen(rl_prompt) + strlen(matches[0]) );
+
+	refresh();
+}
+
+
+/* note: if set, last string MUST be set to NULL */
+static const char * const *completion_values = NULL;
+
+/* readline completion functions, see https://thoughtbot.com/blog/tab-completion-in-gnu-readline */
+
+/* this function is called for each attempted match */
+char *
+getstring_completion_generator(const char *text, int state)
+{
+	static int list_index, len;
+	const char *name;
+
+	if (!state) {
+		list_index = 0;
+		len = strlen(text);
+	}
+
+	while ((name = completion_values[list_index++]) && name!=NULL) {
+		if (strncmp(name, text, len) == 0) {
+			return strdup(name);
+		}
+	}
+
+	return NULL;
+}
+
+/* this function is called when readline attempts completions.  Return a matching function or NULL for no matching */
+char **
+getstring_completion(const char *text, int UNUSED(start), int UNUSED(end))
+{
+	/* do not fall back to default filename completion */
+	rl_attempted_completion_over = 1;
+	rl_completion_append_character = '\0';
+
+	if (completion_values==NULL)
+	{
+		return NULL;
+	}
+	return rl_completion_matches(text, getstring_completion_generator);
+}
+
 #endif
 
+const
+char ** completions_from_tag_table(TagTable * table, size_t num)
+{
+	/* allocate an extra entry at the end, which is set to NULL to terminate the table */
+	const char ** completions = calloc(num+1, sizeof(*completions));
+	for (size_t i=0, j=0; i<num; i++)
+	{
+		if (isalnum(table[i].nodename[0]))
+		{
+			completions[j++] = table[i].nodename;
+		}
+	}
+	return completions;
+}
+
+
 char *
 getstring(char *prompt)
+{
+	return getstring_with_completion(prompt, NULL);
+}
+
+char *
+getstring_with_completion(char *prompt, const char * const * completions)
 {
 	char *buf;
 
 #ifdef HAS_READLINE
+	completion_values = completions;
+	rl_attempted_completion_function = getstring_completion;
+	rl_completion_display_matches_hook = my_rl_completion_display;
 
 	curs_set(1);
+	mymvhline(maxy - 1, 0, ' ', maxx);
 	move(maxy - 1, 0);
 	refresh();
 
 	rl_readline_name = PACKAGE;
-	
+
 	/* set display function for readline to my_rl_display and call readline */
 	rl_redisplay_function = my_rl_display;
 	buf = readline(prompt);
-	if (buf && *buf) 
+	if (buf && *buf)
 		add_history(buf);
-	
+
 	curs_set(0);
 
 #else
@@ -282,7 +406,9 @@ getstring(char *prompt)
 void
 init_curses()
 {
-	initscr();
+	FILE *f = fopen("/dev/tty", "r+");
+	SCREEN *screen = newterm(NULL, f, f);
+	set_term(screen);
 	noecho();
 	cbreak();
 	keypad(stdscr, TRUE);
@@ -290,7 +416,10 @@ init_curses()
 	initcolors();
 	shell_cursor = curs_set(0);
 #ifdef CURSES_MOUSE
-	mousemask(BUTTON1_CLICKED | BUTTON1_DOUBLE_CLICKED, NULL);
+	if (grab_mouse)
+	{
+		mousemask(BUTTON1_CLICKED | BUTTON1_DOUBLE_CLICKED, NULL);
+	}
 #endif
 	curses_open = 1;
 }
@@ -302,7 +431,7 @@ closeprogram()
 	if (curses_open)
 		myendwin();
 	if (ClearScreenAtExit)
-		system("clear");
+		xsystem("clear");
 	else
 		printf("\n");
 	if (tmpfilename1)
@@ -376,10 +505,15 @@ pinfo_getch()
 void
 waitforgetch()
 {
+	int ret;
+
 	fd_set rdfs;
 	FD_ZERO(&rdfs);
 	FD_SET(0, &rdfs);
-	select(1, &rdfs, NULL, NULL, NULL);
+
+	/* we might get interrupted by e.g. SIGTSTP/SIGCONT */
+	do ret = select(1, &rdfs, NULL, NULL, NULL);
+	while (ret == -1 && errno == EINTR);
 }
 
 /* returns 0 on success, 1 on error */
@@ -402,7 +536,7 @@ pinfo_re_comp(char *name)
 		/* compilation failed, so return */
 		return -1;
 	}
-	
+
 	/* compilation succeeded */
 	/* first make some space in h_regexp[] to store the compiled regexp */
 	if (pinfo_re_offset == -1)
@@ -500,9 +634,9 @@ yesno(char *prompt, int def)
 void
 myclrtoeol()
 {
-	int x, y, i;
+	unsigned x, y;
 	getyx(stdscr, y, x);
-	for (i = x; i < maxx; i++)
+	for (unsigned i = x; i < maxx; i++)
 		mvaddch(y, i, ' ');
 }
 
@@ -539,7 +673,7 @@ handlewinch()
 }
 
 /*
- * this functions checks whether the node header node_header 
+ * this functions checks whether the node header node_header
  * corresponds to node node_name
  *
  * e.g. the header is something like:
@@ -553,23 +687,27 @@ handlewinch()
 int
 check_node_name( const char * const node_name, const char * const node_header)
 {
-	/* if either one of node_name or node_header is NULL or a zero 
+	size_t header_len;
+	char *header, *str_start, *c;
+	int res;
+
+	/* if either one of node_name or node_header is NULL or a zero
 	 * sized string, we have nothing to check, so return success */
-	if ( (node_name==NULL) || (node_header==NULL) 
+	if ( (node_name==NULL) || (node_header==NULL)
 		|| (strlen(node_name)==0) || (strlen(node_header)==0) )
 	{
 		return 1;
 	}
 
-	size_t header_len = strlen(node_header);
-	
+	header_len = strlen(node_header);
+
 	/* copy node_header to a local string which can be mutilated */
 	/* don't use strdup here, as xmalloc handles all errors */
-	char *header = xmalloc( header_len + 1 );
+	header = xmalloc( header_len + 1 );
 	strcpy(header, node_header);
 
 	/* search for "Node: foobar," in node_header */
-	char *str_start = strstr(header, "Node: ");
+	str_start = strstr(header, "Node: ");
 	if (str_start==NULL) /* no match */
 	{
 		return 0;
@@ -577,14 +715,14 @@ check_node_name( const char * const node_name, const char * const node_header)
 	/* advance str_start to the start of the node name */
 	str_start += strlen("Node: ");
 	/* and search for the next comma, tab, or newline */
-	char *c = str_start;
+	c = str_start;
 	while ( (*c!=',') && (*c!='\t') && (*c!='\n') && (*c!='\0') ) c++;
 	*c = '\0';
-	
-	/* so, now str_start point to a \0-terminated string containing the 
+
+	/* so, now str_start point to a \0-terminated string containing the
 	 * node name from the header.
 	 * Let's compare it with the node_name we're looking for */
-	int res = strcmp(str_start, node_name);
+	res = strcmp(str_start, node_name);
 
 	/* we're done, so free alloc'ed vars */
 	xfree(header);
@@ -603,7 +741,7 @@ check_node_name( const char * const node_name, const char * const node_header)
 }
 
 
-/* 
+/*
  * The wcswidth function returns the number of columns needed to represent
  * the  wide-character  string pointed to by s, but at most n wide charac‐
  * ters. If a non-printable wide character occurs among these  characters,
@@ -619,7 +757,7 @@ wcswidth(const wchar_t *wstr, size_t max_len)
 
 	/* never count more than max_len chars */
 	if (len>max_len) len=max_len;
-			
+
 	for (i=0; i<len; i++)
 	{
 		if (!iswprint(wstr[i])) return -1;
@@ -631,39 +769,43 @@ wcswidth(const wchar_t *wstr, size_t max_len)
 #endif /* USE_WCHAR && !HAVE_WCSWIDTH */
 
 
-/* calculcate length of string, handling multibyte strings correctly 
+/* calculcate length of string, handling multibyte strings correctly
  * returns value <= len
  */
 int
 width_of_string( const char * const mbs, const int len)
 {
+	int width;
+	char *str;
+#ifdef USE_WCHAR
+	wchar_t *wstr;
+#endif /* USE_WCHAR */
+
 	if (len<0) return -1;
 	if (len==0) return 0;
 
-	int width;
-
-	/* copy the string to a local buffer, because we only want to 
+	/* copy the string to a local buffer, because we only want to
 	 * compare the first len bytes */
-	char *str = xmalloc(len+1);
+	str = xmalloc(len+1);
 	memcpy(str, mbs, len);
-	
+
 #ifdef USE_WCHAR
 
 	/* allocate a widestring */
-	wchar_t *wstr = xmalloc( (len+1)*sizeof(wchar_t) );
-	
+	wstr = xmalloc( (len+1)*sizeof(wchar_t) );
+
 	mbstowcs(wstr, str, len);
 	width = wcswidth(wstr, len);
 
 	/* clean up */
 	xfree(wstr);
-	
+
 #else /* USE_WCHAR */
 
 	width = strlen(str);
-		
+
 #endif /* USE_WCHAR */
-		
+
 	/* clean up */
 	xfree(str);
 
@@ -694,27 +836,27 @@ calculate_len(char *start, char *end)
 	}
 	/* then count everything after the last tab */
 	len += width_of_string(start, c - start);
-	
+
 	return len;
 }
 
-/* 
+/*
  * create a temporary file in a safe way, and return its name in a newly
- * allocated string 
+ * allocated string
  */
 char *
 make_tempfile()
 {
 	char *filename;
 	size_t len;
-	
+
 	/* TODO: fix hardcoded /tmp */
 	char tmpfile_template[] = "/tmp/pinfo.XXXXXX";
 
 	/* create a tmpfile */
 	int fd = mkstemp(tmpfile_template);
 	/* bug out if it failed */
-	if (fd == -1) 
+	if (fd == -1)
 	{
 		closeprogram();
 		printf(_("Couldn't open temporary file\n"));
@@ -725,7 +867,7 @@ make_tempfile()
 	len = strlen(tmpfile_template)+1;
 	filename = xmalloc(len+1); /* guarenteerd to be set to \0's */
 	strncpy(filename, tmpfile_template, len);
-	
+
 	/* close the file */
 	close(fd);
 
