@@ -25,7 +25,7 @@
 extern char *safe_user;
 extern char *safe_group;
 
-#ifndef HAVE_CURS_SET
+#ifndef HAVE_DECL_CURS_SET
 void curs_set (int a);
 #endif
 
@@ -45,6 +45,10 @@ void xfree (void *ptr);
 void *xmalloc (size_t size);
 /* realloc() wrapper */
 void *xrealloc (void *ptr, size_t size);
+/* system(), but return sane error code */
+int system_check(const char *command);
+/* safe, error-checking, command execution */
+void xsystem(const char *command);
 /* initializes GNU locales */
 void initlocale ();
 /* checks if file name does not cause secuirity problems */
@@ -55,6 +59,9 @@ void closeprogram ();
 void init_curses ();
 /* an interface to gnu readline */
 char *getstring (char *prompt);
+char *getstring_with_completion (char *prompt, const char * const *completions);
+/* create a completion table from a tag_table */
+const char ** completions_from_tag_table(TagTable * table, size_t num);
 /* for some reasons mvhline does not work quite properly... */
 void mymvhline (int y, int x, char ch, int len);
 /* this one supports color back/foreground */
@@ -79,7 +86,7 @@ void waitforgetch ();
 extern int curses_open;
 
 /*
- * this functions checks whether the node header node_header 
+ * this functions checks whether the node header node_header
  * corresponds to node node_name
  *
  * returns  0 if node_header does not belong to a node with name node_name
@@ -90,7 +97,7 @@ int
 check_node_name( const char * const node_name, const char * const node_header);
 
 
-/* calculate width of string, handling multibyte encodings 
+/* calculate width of string, handling multibyte encodings
  * correctly */
 int
 width_of_string( const char * const mbs, const int len);
